/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.persistence;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.List;

import org.springframework.security.access.prepost.PreAuthorize;

import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.exception.NewerProjectException;
import kieker.webgui.common.exception.ProjectAlreadyExistingException;
import kieker.webgui.common.exception.ProjectNotExistingException;

import org.primefaces.model.UploadedFile;

/**
 * This is the interface for the data access object(s) which will access for example the file system to manage the available projects.
 * 
 * @author Nils Christian Ehmke
 */
public interface IProjectDAO {

	/**
	 * This method adds a new project to the application. It creates an empty, but nevertheless valid kax-file to the file system. If the method fails due to an
	 * {@link IOException}, it will make sure that the project-directories will be removed as if the method would never have been called.
	 * 
	 * @param projectName
	 *            The name of the new project.
	 * @throws ProjectAlreadyExistingException
	 *             If a project with the same name exists already.
	 * @throws IOException
	 *             If something went wrong during the creation of the project.
	 */
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public abstract void addProject(String projectName) throws ProjectAlreadyExistingException, IOException;

	/**
	 * This method makes a copy of a project and saves it under another name. If the method fails due to an {@link IOException}, it will make sure that the
	 * project-directories of the destination-project will be removed as if the method would never have been called.
	 * 
	 * @param originalProjectName
	 *            The name of the source project.
	 * @param newProjectName
	 *            The name of the target project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given (source) name doesn't exist.
	 * @throws ProjectAlreadyExistingException
	 *             If a project with the same (target) name exists already.
	 * @throws IOException
	 *             If something went wrong during the creation of the target-project or during the loading of the source-project.
	 */
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public abstract void copyProject(String originalProjectName, String newProjectName) throws ProjectNotExistingException, ProjectAlreadyExistingException,
			IOException;

	/**
	 * This method loads the kax-file for the given project name and delivers an initializes instance of {@link MIProject}.
	 * 
	 * @param projectName
	 *            The name of the project to be loaded.
	 * @return The model instance as defined by the corresponding kax-file.
	 * @throws ProjectNotExistingException
	 *             If a project with the given (source) name doesn't exist.
	 * @throws IOException
	 *             If something went wrong during the opening of the project.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract MIProject openProject(String projectName) throws ProjectNotExistingException, IOException;

	/**
	 * This method loads the kax-file for the given project name and delivers an initializes instance of {@link MIProject} - but instead of using the "normal" class
	 * loader, it uses the methods and classes stored in the given instance of {@link ClassAndMethodContainer}. This means that this method <b>does</b> return an
	 * instance of {@link MIProject}, but the one defined in the container. This is also the reason why this method has to return an {@link Object}-instance.
	 * 
	 * @param projectName
	 *            The name of the project to be loaded.
	 * @param classAndMethodContainer
	 *            The container, which will be used to load the project instance.
	 * @return The model instance as defined by the corresponding kax-file.
	 * @throws ProjectNotExistingException
	 *             If a project with the given (source) name doesn't exist.
	 * @throws IOException
	 *             If something went wrong during the opening of the project. This can also mean that the given {@link ClassAndMethodContainer} is somehow invalid.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract Object openProject(String projectName, ClassAndMethodContainer classAndMethodContainer) throws ProjectNotExistingException, IOException;

	/**
	 * This method tries to save the given model instance for the given project. The given time stamp will be compared (if the corresponding flag says so) with the
	 * current time stamp of the project. If the project on the file system has been modified in the meantime, a {@link NewerProjectException} will be thrown. If
	 * something goes wrong during the storage, it is <b>not</b> guaranteed that the resulting file will be valid.
	 * 
	 * @param projectName
	 *            The name of the project which has to be saved.
	 * @param project
	 *            The model instance to be stored in the corresponding kax-file.
	 * @param timeStamp
	 *            The time stamp which has to be compared with the "real" time stamp of the project.
	 * @param overwriteNewerProject
	 *            Determines whether a newer project file will be overwritten without further warning or not-
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws IOException
	 *             If something went wrong during the storage of the model instance.
	 * @throws NewerProjectException
	 *             If the project on the file system is newer and the overwriteNewerProject-flag has not been set.
	 */
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public abstract void saveProject(String projectName, MIProject project, long timeStamp, boolean overwriteNewerProject) throws
			ProjectNotExistingException, IOException, NewerProjectException;

	/**
	 * Delivers the current time stamp of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose time stamp will be delivered.
	 * @return The current time stamp.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract long getCurrTimeStamp(String projectName) throws ProjectNotExistingException;

	/**
	 * This method tries to upload a dependency to the given project. AN existing version will be overwritten.
	 * 
	 * @param file
	 *            The file to be uploaded to the project.
	 * @param projectName
	 *            The name of the project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws IOException
	 *             If something went wrong during the uploading.
	 */
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public abstract void uploadLibrary(UploadedFile file, String projectName) throws ProjectNotExistingException, IOException;

	/**
	 * This method delivers a class loader containing the currently available libraries of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @param requester
	 *            The requester of the classloader.
	 * @return A class loader for the given project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws IOException
	 *             If something went wrong during the initialization of the class loader.
	 */
	@PreAuthorize("isAuthenticated()")
	public ClassLoader getClassLoader(final String projectName, final Object requester) throws ProjectNotExistingException, IOException;

	/**
	 * This method lists all available libraries of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose libraries have to be delivered.
	 * @return A list containing all available library-names of the project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract List<String> listAllLibraries(String projectName) throws ProjectNotExistingException;

	/**
	 * This method lists all available projects on the file system.
	 * 
	 * @return A list containing all available project names.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract Collection<String> listAllProjects();

	/**
	 * This method can be used to deliver the fully qualified URL of a given dependency for a given project.
	 * 
	 * @param lib
	 *            The library whose URL should be delivered.
	 * @param project
	 *            The corresponding project of the library.
	 * @return The URL to the given library if everything went well.
	 * @throws MalformedURLException
	 *             If the URL is for some reason invalid.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract URL getURL(MIDependency lib, String project) throws MalformedURLException;

	/**
	 * Delivers the {@link URL}-element pointing to the kieker library.
	 * 
	 * @return The kieker library.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract URL getKiekerURL();

	/**
	 * Delivers the kax-file for the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return The kax-file of the project.
	 */
	@PreAuthorize("isAuthenticated()")
	public abstract File getProjectFile(String projectName);

	/**
	 * Deletes the given library of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @param libName
	 *            The name of the library.
	 * @return true if and only if the given library has been removed.
	 */
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public abstract boolean deleteLibrary(String projectName, String libName);

}
