/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.persistence.impl;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.ref.WeakReference;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.PostConstruct;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.FileSystemUtils;
import org.springframework.util.WeakReferenceMonitor;
import org.springframework.util.WeakReferenceMonitor.ReleaseListener;

import com.google.common.io.Files;

import kieker.analysis.AnalysisController;
import kieker.analysis.model.analysisMetaModel.MIAnalysisMetaModelFactory;
import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.impl.MAnalysisMetaModelFactory;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.exception.NewerProjectException;
import kieker.webgui.common.exception.ProjectAlreadyExistingException;
import kieker.webgui.common.exception.ProjectNotExistingException;
import kieker.webgui.persistence.IProjectDAO;
import kieker.webgui.persistence.impl.util.CloseableURLClassLoader;

import org.primefaces.model.UploadedFile;

/**
 * This is an implementation of the {@link IProjectDAO} interface, which uses the file system to store the available projects and everything.
 * 
 * @author Nils Christian Ehmke
 */
@Service
public class FSProjectDAOImpl implements IProjectDAO, ReleaseListener {

	private static final Log LOG = LogFactory.getLog(FSProjectDAOImpl.class);

	private static final String KAX_EXTENSION = "kax";
	private static final String LIB_EXTENSION = "jar";
	private static final String LIB_DIRECTORY = "lib";
	private static final String ROOT_DIRECTORY = "data";

	/**
	 * The library for kieker which is contained in the war-file as a resource.
	 */
	private static final String KIEKER_LIB = "kieker-1.6_emf.jar";

	private final MIAnalysisMetaModelFactory factory = new MAnalysisMetaModelFactory();
	private final Map<CloseableURLClassLoader, WeakReference<Object>> classLoaders = new ConcurrentHashMap<CloseableURLClassLoader, WeakReference<Object>>();
	private final Map<File, WeakReference<Object>> tempDirs = new ConcurrentHashMap<File, WeakReference<Object>>();

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public FSProjectDAOImpl() {
		// no code necessary
	}

	/**
	 * This method initializes the object. <b>Do not call this method manually. It will only be accessed by Spring.</b>
	 * 
	 * @throws IOException
	 *             If the creation of the necessary directories failed.
	 */
	@PostConstruct
	public void initialize() throws IOException {
		// Check for the necessary directories and create them if necessary
		this.checkDir(FSProjectDAOImpl.ROOT_DIRECTORY);
	}

	/**
	 * Checks whether the given directory exists and creates it if necessary.
	 * 
	 * @param dirName
	 *            The name of the directory.
	 * @throws IOException
	 *             If the directory could not be created.
	 */
	private void checkDir(final String dirName) throws IOException {
		final File dir = new File(dirName);
		if (!dir.exists() && !dir.mkdir()) {
			// Try to create the directory
			throw new IOException("Could not create directory '" + dirName + "'.");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#addProject(java.lang.String)
	 */
	@Override
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public void addProject(final String projectName) throws ProjectAlreadyExistingException, IOException {
		// Assemble all necessary paths and files for the given project
		final File projectDir = this.assembleProjectDir(projectName);
		final File projectFile = this.assembleKaxFile(projectName);
		final File libDir = this.assembleLibDir(projectName);

		// We need an "empty" project in order to save it.
		final MIProject emptyProject = this.factory.createProject();

		// Make sure that the project doesn't exist already
		if (projectDir.exists()) {
			throw new ProjectAlreadyExistingException("The project with the name '" + projectName + "' exists already.");
		}

		try {
			// Try to create the directories
			if (projectDir.mkdir() && libDir.mkdir()) {
				// Try to save the file
				AnalysisController.saveToFile(projectFile, emptyProject);
			} else {
				// The directories could not be created
				throw new IOException("Project-Directories could not be created.");
			}
		} catch (final IOException ex) {
			// Something went wrong. Remove the directories and files!
			final boolean libDirDeleted = libDir.delete();
			// Keep in mind that the potential remains of the file have to be deleted before the directory.
			final boolean projectFileDeleted = projectFile.delete();
			final boolean projectDeleted = projectDir.delete();

			// The following part is only necessary to calm FindBugs...
			@SuppressWarnings("unused")
			final boolean deleteResults = libDirDeleted && projectFileDeleted && projectDeleted;

			// Rethrow the exception in order to inform the caller of this method
			throw new IOException("An IO Exception occured while saving the project.", ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#copyProject(java.lang.String, java.lang.String)
	 */
	@Override
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public void copyProject(final String originalProjectName, final String newProjectName) throws ProjectNotExistingException, ProjectAlreadyExistingException,
			IOException {
		// Get the necessary paths
		final File dstProjDir = this.assembleProjectDir(newProjectName);
		final File srcLibDir = this.assembleLibDir(originalProjectName);
		final File dstLibDir = this.assembleLibDir(newProjectName);

		// Check whether the project exists already!
		if (this.projectExists(newProjectName)) {
			throw new ProjectAlreadyExistingException("A project with the name '" + newProjectName + "' exists already.");
		}

		final File srcKaxFile = this.assembleKaxFile(originalProjectName);
		final File dstKaxFile = this.assembleKaxFile(newProjectName);

		try {
			if (dstProjDir.mkdir() && dstLibDir.mkdir()) {
				// Copy the kax file
				FileCopyUtils.copy(srcKaxFile, dstKaxFile);

				// Copy the libs
				for (final File lib : srcLibDir.listFiles()) {
					final File dstLibFile = new File(dstLibDir, lib.getName());
					FileCopyUtils.copy(lib, dstLibFile);
				}
			} else {
				throw new IOException("Could not create directories.");
			}
		} catch (final IOException ex) {
			// Something went wrong. Remove the directories and files!
			final boolean libDirDeleted = dstLibDir.delete();
			// Keep in mind that the potential remains of the file have to be deleted before the directory.
			final boolean projectFileDeleted = dstKaxFile.delete();
			final boolean projectDeleted = dstProjDir.delete();

			// The following part is only necessary to calm FindBugs...
			@SuppressWarnings("unused")
			final boolean deleteResults = libDirDeleted && projectFileDeleted && projectDeleted;

			// Rethrow the exception in order to inform the caller of this method
			throw new IOException("An IO Exception occured while copying the project.", ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#openProject(java.lang.String)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public MIProject openProject(final String projectName) throws ProjectNotExistingException, IOException {
		if (projectName == null) {
			throw new IOException("Project is null");
		}

		try {
			// Load the project
			return AnalysisController.loadFromFile(this.assembleKaxFile(projectName).getAbsoluteFile());
		} catch (final FileNotFoundException ex) {
			throw new ProjectNotExistingException("A project with the name '" + projectName + "' does not exist.", ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#openProject(java.lang.String, kieker.webgui.common.ClassAndMethodContainer)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public Object openProject(final String projectName, final ClassAndMethodContainer classAndMethodContainer) throws ProjectNotExistingException, IOException {
		if (projectName == null) {
			throw new IOException("Project is null");
		}

		// Load the project
		final Object project = ClassAndMethodContainer.invokeClassMethod(classAndMethodContainer.getAnalysisControllerLoadFromFile(), null,
				this.assembleKaxFile(projectName));

		if (project == null) {
			throw new IOException("Project could not be loaded.");
		} else {
			return project;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#saveProject(java.lang.String, kieker.analysis.model.analysisMetaModel.MIProject, long, boolean)
	 */
	@Override
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public void saveProject(final String projectName, final MIProject project, final long timeStamp, final boolean overwriteNewerProject) throws
			ProjectNotExistingException, IOException, NewerProjectException {
		// Check whether the project exists
		if (!this.projectExists(projectName)) {
			throw new ProjectNotExistingException("A project with the name '" + projectName + "' does not exist.");
		}

		// Check for a newer version first
		final long currTimeStamp = this.getCurrTimeStamp(projectName);
		if (!overwriteNewerProject && (currTimeStamp > timeStamp)) {
			throw new NewerProjectException("The project with the name '" + projectName + "' has a newer version on the FS.");
		}

		// Try to save it.
		AnalysisController.saveToFile(this.assembleKaxFile(projectName), project);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#getCurrTimeStamp(java.lang.String)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public long getCurrTimeStamp(final String projectName) throws ProjectNotExistingException {
		// Check whether the project exists
		if (!this.projectExists(projectName)) {
			throw new ProjectNotExistingException("A project with the name '" + projectName + "' does not exist.");
		}
		return this.assembleKaxFile(projectName).lastModified();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#uploadLibrary(org.primefaces.model.UploadedFile, java.lang.String)
	 */
	@Override
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public void uploadLibrary(final UploadedFile file, final String projectName) throws ProjectNotExistingException, IOException {
		// Check whether the project exists
		if (!this.projectExists(projectName)) {
			throw new ProjectNotExistingException("A project with the name '" + projectName + "' does not exist.");
		}

		// Prepare the files
		final File libDir = this.assembleLibDir(projectName);
		final File dstFile = new File(libDir, file.getFileName());

		// Get the streams
		final BufferedInputStream in = new BufferedInputStream(file.getInputstream());
		final BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(dstFile));

		// Copy the data - the streams will be closed in this method.
		FileCopyUtils.copy(in, out);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#getClassLoader(java.lang.String)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public ClassLoader getClassLoader(final String projectName, final Object requester) throws ProjectNotExistingException, IOException {
		// Check whether the project exists
		if (!this.projectExists(projectName)) {
			throw new ProjectNotExistingException("A project with the name '" + projectName + "' does not exist.");
		}

		// Create a new temporary directory
		final File tempDir = this.createTemporaryDirectory();
		final List<URL> libs = new ArrayList<URL>();

		// Copy and collect all libraries of the project
		final File libDir = new File(FSProjectDAOImpl.ROOT_DIRECTORY + File.separator + projectName + File.separator + FSProjectDAOImpl.LIB_DIRECTORY);
		final File[] files = libDir.listFiles();
		if (files != null) {
			for (final File file : files) {
				if (file.getName().endsWith("." + FSProjectDAOImpl.LIB_EXTENSION)) {
					final File newLibFile = new File(tempDir, file.getName());
					FileCopyUtils.copy(file, newLibFile);
					try {
						libs.add(newLibFile.toURI().toURL());
					} catch (final MalformedURLException ex) {
						ex.printStackTrace();
					}
				}
			}
		}

		// Add the kieker lib!
		libs.add(this.getKiekerURL());

		// Now assemble the URL class loader
		final PrivilegedClassLoaderAction action = new PrivilegedClassLoaderAction(libs);
		final CloseableURLClassLoader classLoader = AccessController.doPrivileged(action);

		// Remember the requester
		final WeakReference<Object> ref = new WeakReference<Object>(requester);
		this.classLoaders.put(classLoader, ref);
		this.tempDirs.put(tempDir, ref);
		WeakReferenceMonitor.monitor(requester, this);

		return classLoader;
	}

	/**
	 * This method tries to create a new temporary directory within our temp directory.
	 * 
	 * @return A file object pointing to the new temporary directory.
	 */
	private File createTemporaryDirectory() {
		return Files.createTempDir();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#listAllLibraries(java.lang.String)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public List<String> listAllLibraries(final String projectName) throws ProjectNotExistingException {
		// Check whether the project exists
		if (!this.projectExists(projectName)) {
			throw new ProjectNotExistingException("A project with the name '" + projectName + "' does not exist.");
		}

		final List<String> result = new ArrayList<String>();

		// Run through the libs and put them into our list.
		final File[] files = this.assembleLibDir(projectName).listFiles();
		if (files != null) {
			for (final File file : files) {
				if (file.getName().endsWith("." + FSProjectDAOImpl.LIB_EXTENSION)) {
					result.add(file.getName());
				}
			}
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#listAllProjects()
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public Collection<String> listAllProjects() {
		final List<String> result = new ArrayList<String>();

		// Get all directories within our root-dir
		final File[] files = new File(FSProjectDAOImpl.ROOT_DIRECTORY).listFiles();
		for (final File file : files) {
			if (file.isDirectory()) {
				result.add(file.getName());
			}
		}

		return result;
	}

	/**
	 * Checks whether a project with the name exists on the file system.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return true if and only if a directory with the name of the project exists in the root dir.
	 */
	private boolean projectExists(final String projectName) {
		return this.assembleKaxFile(projectName).exists();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#getURL(kieker.analysis.model.analysisMetaModel.MIDependency, java.lang.String)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public URL getURL(final MIDependency lib, final String project) throws MalformedURLException {
		final File file = new File(FSProjectDAOImpl.ROOT_DIRECTORY + File.separator + project + File.separator + FSProjectDAOImpl.LIB_DIRECTORY + File.separator
				+ lib.getFilePath());
		return file.toURI().toURL();
	}

	/**
	 * Assembles the {@link File}-element pointing to the directory of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return The directory of the project.
	 */
	private File assembleProjectDir(final String projectName) {
		return new File(FSProjectDAOImpl.ROOT_DIRECTORY + File.separator + projectName);
	}

	/**
	 * Assembles the {@link File}-element pointing to the kax-file of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return The kax-file of the project.
	 */
	private File assembleKaxFile(final String projectName) {
		return new File(FSProjectDAOImpl.ROOT_DIRECTORY + File.separator + projectName + File.separator + projectName + "." + FSProjectDAOImpl.KAX_EXTENSION);
	}

	/**
	 * Assembles the {@link File}-element pointing to the library directory of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return The library directory of the project.
	 */
	private File assembleLibDir(final String projectName) {
		return new File(FSProjectDAOImpl.ROOT_DIRECTORY + File.separator + projectName + File.separator + FSProjectDAOImpl.LIB_DIRECTORY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#deleteLibrary(java.lang.String, java.lang.String)
	 */
	@Override
	@PreAuthorize("hasAnyRole('User', 'Administrator')")
	public boolean deleteLibrary(final String projectName, final String libName) {
		final File libDir = this.assembleLibDir(projectName);
		final File libFile = new File(libDir, libName);
		return libFile.delete();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#getKiekerURL()
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public URL getKiekerURL() {
		return Thread.currentThread().getContextClassLoader().getResource(FSProjectDAOImpl.KIEKER_LIB);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see kieker.webgui.persistence.IProjectDAO#getProjectFile(java.lang.String)
	 */
	@Override
	@PreAuthorize("isAuthenticated()")
	public File getProjectFile(final String projectName) {
		return this.assembleKaxFile(projectName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.springframework.util.WeakReferenceMonitor.ReleaseListener#released()
	 */
	@Override
	public void released() {
		synchronized (this.classLoaders) {
			int open = 0;

			final List<CloseableURLClassLoader> toBeClosed = new ArrayList<CloseableURLClassLoader>();
			final List<File> toBeRemoved = new ArrayList<File>();

			// Run through the class loaders and check which of them can be closed
			for (final Entry<CloseableURLClassLoader, WeakReference<Object>> entry : this.classLoaders.entrySet()) {
				if (entry.getValue().get() == null) {
					toBeClosed.add(entry.getKey());
				} else {
					open++;
				}
			}
			for (final Entry<File, WeakReference<Object>> entry : this.tempDirs.entrySet()) {
				if (entry.getValue().get() == null) {
					toBeRemoved.add(entry.getKey());
				}
			}

			for (final CloseableURLClassLoader classLoader : toBeClosed) {
				try {
					classLoader.close();
					this.classLoaders.remove(classLoader);
					FSProjectDAOImpl.LOG.info("Closed classloader (" + classLoader + ")");
				} catch (final IOException ex) {
					FSProjectDAOImpl.LOG.error("Could not close classloader (" + classLoader + ")");
				}
			}
			for (final File tempDir : toBeRemoved) {
				final boolean result = FileSystemUtils.deleteRecursively(tempDir);
				if (result) {
					this.tempDirs.remove(tempDir);
					FSProjectDAOImpl.LOG.info("Removed temporary directory (" + tempDir + ")");
				} else {
					FSProjectDAOImpl.LOG.error("Could not remove temporary directory (" + tempDir + ")");
				}
			}
			FSProjectDAOImpl.LOG.info(open + " classloaders still open.");
		}
	}

	/**
	 * This helper class is responsible for creating a classloader as a privileged action. This is recommended due to the java security manager.
	 * 
	 * @author Nils Christian Ehmke
	 * @version 1.0
	 */
	private static class PrivilegedClassLoaderAction implements PrivilegedAction<CloseableURLClassLoader> {
		/**
		 * The list of libraries used to create the class loader.
		 */
		private final List<URL> libs;

		/**
		 * Creates a new instance of this class using the given parameters.
		 * 
		 * @param libs
		 *            The list of libraries used to create the class loader.
		 */
		public PrivilegedClassLoaderAction(final List<URL> libs) {
			this.libs = libs;
		}

		/**
		 * Runs the action.
		 * 
		 * @return The class loader.
		 */
		@Override
		public CloseableURLClassLoader run() {
			// We use "null" as an explicit parent to make sure that the class loader is completely independent from the web gui class loader.
			return new CloseableURLClassLoader(this.libs.toArray(new URL[this.libs.size()]), null);
		}
	}

}
