/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.service;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

import kieker.analysis.AnalysisController.STATE;
import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.repository.AbstractRepository;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.exception.AnalysisInitializationException;
import kieker.webgui.common.exception.AnalysisStateException;
import kieker.webgui.common.exception.DisplayNotFoundException;
import kieker.webgui.common.exception.LibraryLoadException;
import kieker.webgui.common.exception.NewerProjectException;
import kieker.webgui.common.exception.ProjectAlreadyExistingException;
import kieker.webgui.common.exception.ProjectNotExistingException;

import org.primefaces.model.UploadedFile;

/**
 * This is the interface for a component which is responsible for managing all projects. This means that this interface can be used to create, copy, delete projects
 * as well as to start and stop the corresponding analyses and similar things. The facade should abstract from details like the (memory) locations and from
 * synchronization. Latter should be guaranteed by the facade, which means that multiple requests for the same project do not go wrong.
 * 
 * @author Nils Christian Ehmke
 */
public interface IProjectService {

	/**
	 * This method adds a new project to the application. It creates an empty, but nevertheless valid kax-file to the file system. If the method fails due to an
	 * {@link IOException}, it will make sure that the project-directories will be removed as if the method would never have been called.
	 * 
	 * @param projectName
	 *            The name of the new project.
	 * @throws ProjectAlreadyExistingException
	 *             If a project with the same name exists already.
	 * @throws IOException
	 *             If something went wrong during the creation of the project.
	 */
	public void addProject(final String projectName) throws ProjectAlreadyExistingException, IOException;

	/**
	 * This method makes a copy of a project and saves it under another name. If the method fails due to an {@link IOException}, it will make sure that the
	 * project-directories of the destination-project will be removed as if the method would never have been called.
	 * 
	 * @param originalProjectName
	 *            The name of the source project.
	 * @param newProjectName
	 *            The name of the target project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given (source) name doesn't exist.
	 * @throws ProjectAlreadyExistingException
	 *             If a project with the same (target) name exists already.
	 * @throws IOException
	 *             If something went wrong during the creation of the target-project or during the loading of the source-project.
	 */
	public void copyProject(final String originalProjectName, final String newProjectName) throws ProjectNotExistingException, ProjectAlreadyExistingException,
			IOException;

	/**
	 * This method loads the kax-file for the given project name and delivers an initializes instance of {@link MIProject}.
	 * 
	 * @param projectName
	 *            The name of the project to be loaded.
	 * @return The model instance as defined by the corresponding kax-file.
	 * @throws ProjectNotExistingException
	 *             If a project with the given (source) name doesn't exist.
	 * @throws IOException
	 *             If something went wrong during the opening of the project.
	 */
	public MIProject openProject(final String projectName) throws ProjectNotExistingException, IOException;

	/**
	 * This method loads the kax-file for the given project name and delivers an initializes instance of {@link MIProject} - but instead of using the "normal" class
	 * loader, it uses the methods and classes stored in the given instance of {@link ClassAndMethodContainer}. This means that this method <b>does</b> return an
	 * instance of {@link MIProject}, but the one defined in the container. This is also the reason why this method has to return an {@link Object}-instance.
	 * 
	 * @param projectName
	 *            The name of the project to be loaded.
	 * @param classAndMethodContainer
	 *            The container, which will be used to load the project instance.
	 * @return The model instance as defined by the corresponding kax-file.
	 * @throws ProjectNotExistingException
	 *             If a project with the given (source) name doesn't exist.
	 * @throws IOException
	 *             If something went wrong during the opening of the project. This can also mean that the given {@link ClassAndMethodContainer} is somehow invalid.
	 */
	public Object openProject(final String projectName, final ClassAndMethodContainer classAndMethodContainer) throws ProjectNotExistingException, IOException;

	/**
	 * This method tries to save the given model instance for the given project. The given time stamp will be compared (if the corresponding flag says so) with the
	 * current time stamp of the project. If the project on the file system has been modified in the meantime, a {@link NewerProjectException} will be thrown. If
	 * something goes wrong during the storage, it is <b>not</b> guaranteed that the resulting file will be valid.
	 * 
	 * @param projectName
	 *            The name of the project which has to be saved.
	 * @param project
	 *            The model instance to be stored in the corresponding kax-file.
	 * @param timeStamp
	 *            The time stamp which has to be compared with the "real" time stamp of the project.
	 * @param overwriteNewerProject
	 *            Determines whether a newer project file will be overwritten without further warning or not-
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws IOException
	 *             If something went wrong during the storage of the model instance.
	 * @throws NewerProjectException
	 *             If the project on the file system is newer and the overwriteNewerProject-flag has not been set.
	 */
	public void saveProject(final String projectName, final MIProject project, final long timeStamp, final boolean overwriteNewerProject) throws
			ProjectNotExistingException, IOException, NewerProjectException;

	/**
	 * Delivers the current time stamp of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose time stamp will be delivered.
	 * @return The current time stamp.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 */
	public long getCurrTimeStamp(final String projectName) throws ProjectNotExistingException;

	/**
	 * This method tries to upload a dependency to the given project. An existing version of the library will be overwritten.
	 * 
	 * @param file
	 *            The file to be uploaded to the project.
	 * @param projectName
	 *            The name of the project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws IOException
	 *             If something went wrong during the uploading.
	 */
	public void uploadLibrary(final UploadedFile file, final String projectName) throws ProjectNotExistingException, IOException;

	/**
	 * This method delivers a class loader containing the currently available libraries of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @param requester
	 *            The requester of the class loader.
	 * @return A class loader for the given project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws IOException
	 *             If something went wrong during the initialization of the class loader.
	 */
	public ClassLoader getClassLoader(final String projectName, final Object requester) throws ProjectNotExistingException, IOException;

	/**
	 * This method lists all available libraries of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose libraries have to be delivered.
	 * @return A list containing all available library-names of the project.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 */
	public List<String> listAllLibraries(final String projectName) throws ProjectNotExistingException;

	/**
	 * This method delivers the available classes from the type {@link AbstractRepository} within the given dependency for the given project, using the given
	 * parameters.
	 * 
	 * @param lib
	 *            The library to be searched.
	 * @param project
	 *            The project for the given library.
	 * @param classLoader
	 *            The class loader to be used.
	 * @param classAndMethodContainer
	 *            The container with classes and methods to be used.
	 * @return A list with all repository-classes.
	 * @throws LibraryLoadException
	 *             If something went wrong during the loading of the library.
	 * @throws IOException
	 */
	public List<Class<AbstractRepository>> getAllRepositoriesWithinLib(final MIDependency lib, final String project, final ClassLoader classLoader,
			final ClassAndMethodContainer classAndMethodContainer) throws LibraryLoadException;

	/**
	 * This method delivers the available classes from the type {@link AbstractPlugin} within the given dependency for the given project, using the given
	 * parameters.
	 * 
	 * @param lib
	 *            The library to be searched.
	 * @param project
	 *            The project for the given library.
	 * @param classLoader
	 *            The class loader to be used.
	 * @param classAndMethodContainer
	 *            The container with classes and methods to be used.
	 * @return A list with all plugin-classes.
	 * @throws LibraryLoadException
	 *             If something went wrong during the loading of the library.
	 */
	public List<Class<AbstractPlugin>> getAllPluginsWithinLib(final MIDependency lib, final String project, final ClassLoader classLoader,
			final ClassAndMethodContainer classAndMethodContainer) throws LibraryLoadException;

	/**
	 * This method delivers the available classes from the type {@link AbstractRepository} within the kieker dependency using the given parameters.
	 * 
	 * @param classLoader
	 *            The class loader to be used.
	 * @param classAndMethodContainer
	 *            The container with classes and methods to be used.
	 * @return A list with all repository-classes.
	 * @throws LibraryLoadException
	 *             If something went wrong during the loading of the library.
	 */
	public List<Class<AbstractRepository>> getAllRepositoriesWithinKiekerLib(final ClassLoader classLoader,
			final ClassAndMethodContainer classAndMethodContainer) throws LibraryLoadException;

	/**
	 * This method delivers the available classes from the type {@link AbstractPlugin} within the kieker dependency using the given parameters.
	 * 
	 * @param classLoader
	 *            The class loader to be used.
	 * @param classAndMethodContainer
	 *            The container with classes and methods to be used.
	 * @return A list with all plugin-classes.
	 * @throws LibraryLoadException
	 *             If something went wrong during the loading of the library.
	 */
	public List<Class<AbstractPlugin>> getAllPluginsWithinKiekerLib(final ClassLoader classLoader, final ClassAndMethodContainer classAndMethodContainer) throws
			LibraryLoadException;

	/**
	 * This method lists all available projects on the file system.
	 * 
	 * @return A list containing all available project names.
	 */
	public Collection<String> listAllProjects();

	/**
	 * This method initializes the analysis of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose analysis should be initialized.
	 * @param classLoader
	 *            The class loader to be used during the loading.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisStateException
	 *             If the analysis of the given project is in the wrong state to be initialized. This means that it has not been cleaned yet.
	 * @throws AnalysisInitializationException
	 *             If an error occurred during the initialization of the analysis.
	 */
	public void initializeAnalysis(final String projectName, final ClassLoader classLoader) throws ProjectNotExistingException, AnalysisStateException,
			AnalysisInitializationException;

	/**
	 * This method cleans the analysis of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose analysis should be cleaned.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisStateException
	 *             If the analysis of the given project is in the wrong state to be cleaned. This means that it has not been terminated yet.
	 */
	public void cleanAnalysis(final String projectName) throws ProjectNotExistingException, AnalysisStateException;

	/**
	 * This method starts the analysis of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose analysis should be started.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisStateException
	 *             If the analysis of the given project is in the wrong state to be started. This means that it has not been initialized yet.
	 */
	public void startAnalysis(final String projectName) throws ProjectNotExistingException, AnalysisStateException;

	/**
	 * This method stops the analysis of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose analysis should be stopped.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisStateException
	 *             If the analysis of the given project is in the wrong state to be stopped. This means that it has not been started yet or is no longer running.
	 */
	public void stopAnalysis(final String projectName) throws ProjectNotExistingException, AnalysisStateException;

	/**
	 * This method delivers the display object of the (currently running) analysis for the given project and the given parameters. Technically it is an instance of
	 * {@code AbstractDisplay}, but in fact the project specific class loader has been used.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @param viewName
	 *            The name of the view.
	 * @param displayName
	 *            The name of the display.
	 * @return A display object for the given parameters.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws DisplayNotFoundException
	 *             If a view or a display within the given view does not exist.
	 */
	public Object getDisplay(final String projectName, final String viewName, final String displayName) throws ProjectNotExistingException, DisplayNotFoundException;

	/**
	 * This method delivers the current state of the given project.
	 * 
	 * @param projectName
	 *            The name of the project whose state have to be returned.
	 * @return The state of the given project, if available. {@code null} otherwise.
	 */
	public STATE getCurrentState(final String projectName);

	/**
	 * This method delivers the available log entries of the analysis controller of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return An array containing the entries of the log.
	 * @throws AnalysisStateException
	 *             If the analysis is in an invalid state to deliver the entries.
	 */
	public Object[] getLogEntries(final String projectName) throws AnalysisStateException;

	/**
	 * Deletes the given library of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @param libName
	 *            The name of the library.
	 * @return true if and only if the given library has been removed.
	 */
	public boolean deleteLibrary(String projectName, String libName);
}
