/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.service.impl.util;

import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import kieker.analysis.AnalysisController.STATE;
import kieker.webgui.common.exception.AnalysisInitializationException;
import kieker.webgui.common.exception.AnalysisStateException;
import kieker.webgui.common.exception.DisplayNotFoundException;
import kieker.webgui.common.exception.ProjectNotExistingException;
import kieker.webgui.persistence.IProjectDAO;

/**
 * This manager is responsible for the currently used and running instances of {@code AnalysisController}. It supplies methods to check the states of analysis,
 * instantiate, start and to stop them. This is also a singleton instance.
 * 
 * @author Nils Christian Ehmke
 */
@Service
public final class ACManager {
	/**
	 * This list contains the current analysis controllers and their corresponding threads. Not every project does have a controller, but every project can have
	 * maximal one.
	 */
	private final ConcurrentHashMap<String, Analysis> analyses = new ConcurrentHashMap<String, Analysis>();
	@Autowired
	private IProjectDAO projectDAO;

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public ACManager() {
		// No code necessary.
	}

	/**
	 * This method initializes the analysis for the given project.
	 * 
	 * @param projectName
	 *            The name of the project to be initialized.
	 * @param classLoader
	 *            The class loader to be used during the loading.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisInitializationException
	 *             If an error occurred during the initialization of the analysis.
	 * @throws AnalysisStateException
	 *             If the analysis is in an invalid state to be initialized.
	 */
	public void initializeAnalysis(final String projectName, final ClassLoader classLoader) throws ProjectNotExistingException, AnalysisStateException,
			AnalysisInitializationException {
		// The analysis for the given project must not exist!
		if (this.analyses.containsKey(projectName)) {
			throw new AnalysisStateException("The analysis has not been cleaned yet.");
		}

		final Analysis analysis = new Analysis(classLoader, this.projectDAO.getProjectFile(projectName));
		this.analyses.put(projectName, analysis);
	}

	/**
	 * This method cleans the analysis for the given project.
	 * 
	 * @param projectName
	 *            The name of the project to be cleaned.
	 * @throws AnalysisStateException
	 *             If the analysis is in an invalid state to be cleaned.
	 */
	public void cleanAnalysis(final String projectName) throws AnalysisStateException {
		// The analysis for the given project must exist!
		if (!this.analyses.containsKey(projectName)) {
			// Nothing to do
			return;
		}
		final Analysis analysis = this.analyses.get(projectName);

		// The analysis for the given project must not run
		if (STATE.RUNNING.toString().equals(analysis.getCurrentState().toString())) {
			throw new AnalysisStateException("The analysis is still running.");
		}

		this.analyses.remove(projectName);
	}

	/**
	 * This method starts the analysis for the given project.
	 * 
	 * @param projectName
	 *            The name of the project to be started.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisStateException
	 *             If the analysis is in an invalid state to be started.
	 */
	public void startAnalysis(final String projectName) throws ProjectNotExistingException, AnalysisStateException {
		// The analysis for the given project must exist!
		if (!this.analyses.containsKey(projectName)) {
			throw new AnalysisStateException("The analysis has not been initialized yet.");
		}
		final Analysis analysis = this.analyses.get(projectName);

		// The analysis for the given project must be ready
		if (!STATE.READY.toString().equals(analysis.getCurrentState().toString())) {
			throw new AnalysisStateException("The analysis is not ready.");
		}

		analysis.start();
	}

	/**
	 * This method stops the analysis for the given project.
	 * 
	 * @param projectName
	 *            The name of the project to be stopped.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws AnalysisStateException
	 *             If the analysis is in an invalid state to be stopped.
	 */
	public void stopAnalysis(final String projectName) throws ProjectNotExistingException, AnalysisStateException {
		// The analysis for the given project must exist!
		if (!this.analyses.containsKey(projectName)) {
			throw new AnalysisStateException("The analysis has not been initialized yet.");
		}
		final Analysis analysis = this.analyses.get(projectName);

		// The analysis for the given project must be running
		if (!STATE.RUNNING.toString().equals(analysis.getCurrentState().toString())) {
			throw new AnalysisStateException("The analysis is not running.");
		}

		analysis.stop();
	}

	/**
	 * This method delivers the available log entries of the analysis controller of the given project.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return An array containing the entries of the log.
	 * @throws AnalysisStateException
	 *             If the analysis is in an invalid state to deliver the entries.
	 */
	public Object[] getLogEntries(final String projectName) throws AnalysisStateException {
		// The analysis for the given project must exist!
		if (!this.analyses.containsKey(projectName)) {
			throw new AnalysisStateException("The analysis has not been initialized yet.");
		}
		return this.analyses.get(projectName).getLogEntries();
	}

	/**
	 * This method delivers the display object of the (currently running) analysis for the given project and the given parameters. Technically it is an instance of
	 * {@code AbstractDisplay}, but in fact the project specific class loader has been used.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @param viewName
	 *            The name of the view.
	 * @param displayName
	 *            The name of the display.
	 * @return A display object for the given parameters.
	 * @throws ProjectNotExistingException
	 *             If a project with the given name does not exist.
	 * @throws DisplayNotFoundException
	 *             If a view or a display within the given view does not exist.
	 */
	public Object getDisplay(final String projectName, final String viewName, final String displayName) throws ProjectNotExistingException,
			DisplayNotFoundException {
		// The analysis for the given project must exist!
		if (!this.analyses.containsKey(projectName)) {
			throw new ProjectNotExistingException("The analysis has not been initialized yet.");
		}

		return this.analyses.get(projectName).getDisplay(viewName, displayName);
	}

	/**
	 * This method delivers the current state of the analysis, if it is available, null otherwise.
	 * 
	 * @param projectName
	 *            The name of the project.
	 * @return The state of the given project.
	 */
	public STATE getCurrentState(final String projectName) {
		STATE retState = null;

		// The analysis for the given project must exist!
		if (this.analyses.containsKey(projectName)) {
			final Analysis analysis = this.analyses.get(projectName);
			final Enum<?> state = analysis.getCurrentState();

			if (state != null) {
				if (STATE.FAILED.toString().equals(state.toString())) { // NOPMD (deeply nested if)
					retState = STATE.FAILED;
				}
				if (STATE.READY.toString().equals(state.toString())) { // NOPMD (deeply nested if)
					retState = STATE.READY;
				}
				if (STATE.RUNNING.toString().equals(state.toString())) { // NOPMD (deeply nested if)
					retState = STATE.RUNNING;
				}
				if (STATE.TERMINATED.toString().equals(state.toString())) { // NOPMD (deeply nested if)
					retState = STATE.TERMINATED;
				}
			}
		}
		return retState;
	}

}
