/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.service.impl.util;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import kieker.analysis.AnalysisController;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.exception.AnalysisInitializationException;
import kieker.webgui.common.exception.AnalysisStateException;
import kieker.webgui.common.exception.ProjectLoadException;

/**
 * An analysis within the web gui.
 * 
 * @author Nils Christian Ehmke
 */
public final class Analysis {
	private static final Log LOG = LogFactory.getLog(Analysis.class);
	private static final long MAX_THREAD_WAIT_TIME_MS = 1000;
	private final ClassAndMethodContainer classAndMethodContainer;
	private final Object analysisController;
	private final Object analysisControllerThread;
	private final UpdateDisplaysThread updateDisplaysThread;

	/**
	 * Creates a new instance of this class using the given parameters.
	 * 
	 * @param classLoader
	 *            The class loader to be used to initialize the analysis.
	 * @param projectFile
	 *            The file to be loaded.
	 * @throws AnalysisStateException
	 *             If something went wrong during the loading of the analysis.
	 * @throws AnalysisInitializationException
	 *             If an error occurred during the instantiation of the analysis.
	 */
	public Analysis(final ClassLoader classLoader, final File projectFile) throws AnalysisStateException, AnalysisInitializationException {
		try {
			this.classAndMethodContainer = new ClassAndMethodContainer(classLoader);

			final Method createMethod = this.classAndMethodContainer.getAnalysisControllerCreateAnalysisController();
			final Object modelProject = ClassAndMethodContainer.invokeClassMethod(this.classAndMethodContainer.getAnalysisControllerLoadFromFile(), null,
					projectFile.getAbsoluteFile());
			final Object controllerAndMapping = ClassAndMethodContainer.invokeClassMethod(createMethod, null, modelProject, classLoader);
			this.analysisController = ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getAnalysisControllerWithMappingGetController(),
					controllerAndMapping, null);
			this.analysisControllerThread = ClassAndMethodContainer.invokeConstructor(this.classAndMethodContainer.getAnalysisControllerThreadConstructor(),
					this.analysisController);
			this.updateDisplaysThread = new UpdateDisplaysThread(); // ClassAndMethodContainer.invokeMethod(
			// this.classAndMethodContainer.getAnalysisControllerWithMappingGetMapping(), controllerAndMapping, null));
			if ((this.analysisController == null) || (this.analysisControllerThread == null)) {
				throw new AnalysisInitializationException("An error occured while instantiating the analysis.");
			}
		} catch (final ProjectLoadException ex) {
			throw new AnalysisInitializationException("An error occured while instantiating the analysis.", ex);
		} catch (final NullPointerException ex) {
			throw new AnalysisInitializationException("An error occured while instantiating the analysis.", ex);
		}
	}

	/**
	 * Starts the analysis.
	 * 
	 * @throws AnalysisStateException
	 *             If the analysis is in the wrong state to be started.
	 */
	public void start() throws AnalysisStateException {
		synchronized (this) {
			try {
				this.classAndMethodContainer.getAnalysisControllerThreadStart().invoke(this.analysisControllerThread, new Object[0]);
				this.updateDisplaysThread.start();
			} catch (final IllegalThreadStateException ex) {
				throw new AnalysisStateException("The analysis has already been started once. It has to be reinitialized to be runnable again.", ex);
			} catch (final IllegalAccessException ex) {
				Analysis.LOG.error("An error occured during a reflection method call.", ex);
			} catch (final IllegalArgumentException ex) {
				Analysis.LOG.error("An error occured during a reflection method call.", ex);
			} catch (final InvocationTargetException ex) {
				throw new AnalysisStateException("The analysis has already been started once. It has to be reinitialized to be runnable again.", ex);
			}
		}

	}

	/**
	 * Stops the analysis.
	 */
	public void stop() {
		synchronized (this) {
			try {
				this.classAndMethodContainer.getAnalysisControllerThreadTerminate().invoke(this.analysisControllerThread, new Object[0]);
				this.updateDisplaysThread.terminate();

				this.classAndMethodContainer.getAnalysisControllerThreadJoin().invoke(this.analysisControllerThread, Analysis.MAX_THREAD_WAIT_TIME_MS);
				this.updateDisplaysThread.join(Analysis.MAX_THREAD_WAIT_TIME_MS);
			} catch (final InterruptedException ex) {
				// Log the exception, but don't handle it further
				Analysis.LOG.info("An interrupted exception occured.", ex);
			} catch (final IllegalAccessException ex) {
				Analysis.LOG.error("An error occured during a reflection method call.", ex);
			} catch (final IllegalArgumentException ex) {
				Analysis.LOG.error("An error occured during a reflection method call.", ex);
			} catch (final InvocationTargetException ex) {
				Analysis.LOG.error("An error occured during a reflection method call.", ex);
			}
		}
	}

	/**
	 * Delivers the current state of the analysis.
	 * 
	 * @return The state.
	 */
	public Enum<?> getCurrentState() {
		synchronized (this) {
			return (Enum<?>) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getAnalysisControllerGetState(), this.analysisController, null);
		}
	}

	/**
	 * This method delivers the display object of the (currently running) analysis. Technically it is an instance of {@code AbstractDisplay}, but in fact the project
	 * specific class loader has been used.
	 * 
	 * @param viewName
	 *            The name of the view.
	 * @param displayName
	 *            The name of the display.
	 * @return A display object for the given parameters.
	 */
	public Object getDisplay(final String viewName, final String displayName) {
		return this.updateDisplaysThread.getDisplay(viewName, displayName);
	}

	/**
	 * Delivers the log entries for the current analysis.
	 * 
	 * @return An array containing the log entries (if available).
	 */
	public Object[] getLogEntries() {
		return (Object[]) ClassAndMethodContainer.invokeClassMethod(Analysis.this.classAndMethodContainer.getLogImplWebguiLoggingClassGetEntriesMethod(), null,
				AnalysisController.class.getName());
	}

	/**
	 * This manager is responsible for the currently used and running instances of {@code AnalysisController}. It supplies methods to check the states of analysis,
	 * instantiate, start and to stop them.
	 * 
	 * @author Nils Christian Ehmke
	 * @version 1.0
	 */
	private static class UpdateDisplaysThread extends Thread {

		private static final long SLEEP_TIME_MS = 2 * 1000;
		// private final Object pluginMap;
		private final Map<String, Map<String, Object>> displayObjects = new ConcurrentHashMap<String, Map<String, Object>>(); // NOPMD
		// private final Map<Object, Method> methodMap = new ConcurrentHashMap<Object, Method>(); // NOPMD (Use of concurrent hash map)
		private volatile boolean terminated = false;

		/**
		 * Default constructor.
		 */
		public UpdateDisplaysThread() {
			// this.pluginMap = object;
		}

		/**
		 * Delivers the correct method to be invoked for an update of the display.
		 * 
		 * @param clazz
		 *            The class of the plugin.
		 * @param displayName
		 *            The name within the display-annotation.
		 * @return The method instance.
		 */
		/*
		 * private Method getDisplayMethod(final Class<? extends AbstractPlugin> clazz, final String displayName) {
		 * final Method[] methods = clazz.getMethods();
		 * for (final Method method : methods) {
		 * try {
		 * final Annotation displayAnnot = method.getAnnotation(Analysis.this.classAndMethodContainer.getDisplayAnnotationClass());
		 * if ((displayAnnot != null)
		 * && (Analysis.this.classAndMethodContainer.getDisplayNameMethod().invoke(displayAnnot, new Object[0]).equals(displayName))) {
		 * // We found the correct method
		 * return method;
		 * }
		 * } catch (final IllegalAccessException ex) {
		 * // Log this exception, but ignore it
		 * Analysis.LOG.info("An error occured while calling a method.", ex);
		 * } catch (final IllegalArgumentException ex) {
		 * // Log this exception, but ignore it
		 * Analysis.LOG.info("An error occured while calling a method.", ex);
		 * } catch (final InvocationTargetException ex) {
		 * // Log this exception, but ignore it
		 * Analysis.LOG.info("An error occured while calling a method.", ex);
		 * }
		 * }
		 * return null;
		 * }
		 */

		/**
		 * Delivers the display object for the given view and the given display.
		 * 
		 * @param viewName
		 *            The name of the view.
		 * @param displayName
		 *            The name of the display in the view.
		 * @return The object for the given display if it exists, null otherwise.
		 */
		public Object getDisplay(final String viewName, final String displayName) {
			return this.displayObjects.get(viewName).get(displayName);
		}

		@Override
		public void run() {
			// Run until we final have been interrupted
			while (!this.terminated) {
				// Wait a little bit.
				try {
					Thread.sleep(UpdateDisplaysThread.SLEEP_TIME_MS);
				} catch (final InterruptedException ex) {
					// We have been interrupted. Exit the thread
					return;
				}
			}

		}

		/**
		 * This method makes sure that the thread terminates at the next possible point.
		 */
		public void terminate() {
			this.terminated = true;
		}
	}
}
