/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.application;

import java.io.Serializable;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.context.FacesContext;

/**
 * The class {@link GlobalPropertiesBean} is a singleton scoped Spring bean, containing properties, constants and partially localized texts for the application. Some
 * of the properties are injected via Spring, others are loaded directly from the resource bundle. This makes sure that those constants can be modified without a
 * necessary recompiling of the whole application.<br>
 * It implements the {@link Serializable} interface to make sure that some session scoped beans can use this class.<br>
 * It is singleton scoped as the properties are the same for every user and as only one instance of this class is necessary.
 * 
 * @author Nils Christian Ehmke
 */
public final class GlobalPropertiesBean implements Serializable {

	private static final long serialVersionUID = 4253541090603377504L;

	private static final String RESOURCE_BUNDLE_NAME = "lang.Common";

	private static final String PROPERTY_MSG_PROJECT_LOADING_EXCEPTION = "msgProjectLoadingException";
	private static final String PROPERTY_MSG_PROJECT_SAVED = "msgProjectSaved";
	private static final String PROPERTY_MSG_PLUGIN_CREATION_EXCEPTION = "msgPluginCreationException";
	private static final String PROPERTY_MSG_REPOSITORY_CREATION_EXCEPTION = "msgRepositoryCreationException";
	private static final String PROPERTY_MSG_LIBRARY_UPLOADED = "msgLibraryUploaded";
	private static final String PROPERTY_MSG_PROJECT_CREATED = "msgProjectCreated";
	private static final String PROPERTY_MSG_PROJECT_SAVING_EXCEPTION = "msgProjectSavingException";
	private static final String PROPERTY_MSG_PROJECT_NOT_EXISTING_EXCEPTION = "msgProjectNotExistingException";
	private static final String PROPERTY_MSG_PROJECT_MODIFIED = "msgProjectModified";
	private static final String PROPERTY_MSG_LIBRARY_EXISTING_EXCEPTION = "msgLibraryExistingException";
	private static final String PROPERTY_MSG_LIBRARY_UPLOADING_EXCEPTION = "msgLibraryUploadingException";

	private String themeCookieName;
	private String projectOverviewPage;
	private String facesContextThemeKey;
	private String defaultTheme;
	private String analysisEditorGridSizeCookieName;
	private String analysisEditorGridColorCookieName;
	private String analysisEditorDefaultGridSize;
	private String analysisEditorDefaultGridColor;

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public GlobalPropertiesBean() {
		// No code necessary
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#projectOverviewPage}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param projectOverviewPage
	 *            The new value for the property.
	 */
	public void setProjectOverviewPage(final String projectOverviewPage) {
		this.projectOverviewPage = projectOverviewPage;
	}

	/**
	 * This method delivers the redirecting-link to the project overview page.
	 * 
	 * @return The value of the property.
	 */
	public String getProjectOverviewPage() {
		return this.projectOverviewPage;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#facesContextThemeKey}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param facesContextThemeKey
	 *            The new value for the property.
	 */
	public void setFacesContextThemeKey(final String facesContextThemeKey) {
		this.facesContextThemeKey = facesContextThemeKey;
	}

	/**
	 * This method delivers the key name of the theme within the faces context. This key can be used to load the default theme from the faces context if necessary.
	 * 
	 * @return The value of the property.
	 */
	public String getFacesContextThemeKey() {
		return this.facesContextThemeKey;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#defaultTheme}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param defaultTheme
	 *            The new value for the property.
	 */
	public void setDefaultTheme(final String defaultTheme) {
		this.defaultTheme = defaultTheme;
	}

	/**
	 * This method delivers the default theme stored in the properties file.
	 * 
	 * @return The value of the property.
	 */
	public String getDefaultTheme() {
		return this.defaultTheme;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#analysisEditorGridSizeCookieName}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param analysisEditorGridSizeCookieName
	 *            The new value for the property.
	 */
	public void setAnalysisEditorGridSizeCookieName(final String analysisEditorGridSizeCookieName) {
		this.analysisEditorGridSizeCookieName = analysisEditorGridSizeCookieName;
	}

	/**
	 * This method delivers the name of the cookie for the grid size within the analysis editor.
	 * 
	 * @return The value of the property.
	 */
	public String getAnalysisEditorGridSizeCookieName() {
		return this.analysisEditorGridSizeCookieName;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#analysisEditorGridColorCookieName}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param analysisEditorGridColorCookieName
	 *            The new value for the property.
	 */
	public void setAnalysisEditorGridColorCookieName(final String analysisEditorGridColorCookieName) {
		this.analysisEditorGridColorCookieName = analysisEditorGridColorCookieName;
	}

	/**
	 * This method delivers the name of the cookie for the grid color within the analysis editor.
	 * 
	 * @return The value of the property.
	 */
	public String getAnalysisEditorGridColorCookieName() {
		return this.analysisEditorGridColorCookieName;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#analysisEditorDefaultGridSize}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param analysisEditorDefaultGridSize
	 *            The new value for the property.
	 */
	public void setAnalysisEditorDefaultGridSize(final String analysisEditorDefaultGridSize) {
		this.analysisEditorDefaultGridSize = analysisEditorDefaultGridSize;
	}

	/**
	 * This method delivers the default grid size within the analysis editor.
	 * 
	 * @return The value of the property.
	 */
	public String getAnalysisEditorDefaultGridSize() {
		return this.analysisEditorDefaultGridSize;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#analysisEditorDefaultGridColor}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param analysisEditorDefaultGridColor
	 *            The new value for the property.
	 */
	public void setAnalysisEditorDefaultGridColor(final String analysisEditorDefaultGridColor) {
		this.analysisEditorDefaultGridColor = analysisEditorDefaultGridColor;
	}

	/**
	 * This method delivers the default grid color within the analysis editor.
	 * 
	 * @return The value of the property.
	 */
	public String getAnalysisEditorDefaultGridColor() {
		return this.analysisEditorDefaultGridColor;
	}

	/**
	 * Setter for the property {@link GlobalPropertiesBean#themeCookieName}. <b>Do not use this method. This property is Spring managed.</b>
	 * 
	 * @param themeCookieName
	 *            The new value for the property.
	 */
	public void setThemeCookieName(final String themeCookieName) {
		this.themeCookieName = themeCookieName;
	}

	/**
	 * This method delivers the name of the theme cookie stored in the properties file. This cookie name should only be used to (re)store the current user's look and
	 * feel.
	 * 
	 * @return The value of the property.
	 */
	public String getThemeCookieName() {
		return this.themeCookieName;
	}

	/**
	 * This method delivers the localized message, which can be used if the loading of a project resulted in an exception.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgProjectLoadingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_LOADING_EXCEPTION);
	}

	/**
	 * This method delivers the localized message, which can be used if a project has been saved.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgProjectSaved() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_SAVED);
	}

	/**
	 * This method delivers the localized message, which can be used if the creation of a plugin resulted in an exception.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgPluginCreationException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PLUGIN_CREATION_EXCEPTION);
	}

	/**
	 * This method delivers the localized message, which can be used if the creation of a repository resulted in an exception.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgRepositoryCreationException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_REPOSITORY_CREATION_EXCEPTION);
	}

	/**
	 * This method delivers the localized message, which can be used if a library has been uploaded.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgLibraryUploaded() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_LIBRARY_UPLOADED);
	}

	/**
	 * This method delivers the localized message, which can be used if a project has been created.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgProjectCreated() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_CREATED);
	}

	/**
	 * This method delivers the localized message, which can be used if the saving of a project failed.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgProjectSavingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_SAVING_EXCEPTION);
	}

	/**
	 * This method delivers the localized message, which can be used if a project does not exist.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgProjectNotExistingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_NOT_EXISTING_EXCEPTION);
	}

	/**
	 * This method delivers the localized message, which can be used if a project has been modified externally.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgProjectModified() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_PROJECT_MODIFIED);
	}

	/**
	 * This method delivers the localized message, which can be used if a library does not exist.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgLibraryExistingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_LIBRARY_EXISTING_EXCEPTION);
	}

	/**
	 * This method delivers the localized message, which can be used if the uploading of a library failed.
	 * 
	 * @return The value of the property.
	 */
	public String getMsgLibraryUploadingException() {
		return GlobalPropertiesBean.getLocalizedString(GlobalPropertiesBean.PROPERTY_MSG_LIBRARY_UPLOADING_EXCEPTION);
	}

	/**
	 * Delivers the localized property for the given key.
	 * 
	 * @param key
	 *            They key of the property.
	 * @return The value for the current locale (in the current faces context) from the common resource bundle.
	 */
	private static String getLocalizedString(final String key) {
		// Get the correct resource bundle for the current language (taken from the current context) and search it for the given key
		return ResourceBundle.getBundle(GlobalPropertiesBean.RESOURCE_BUNDLE_NAME, FacesContext.getCurrentInstance().getELContext().getLocale()).getString(key);
	}

	/**
	 * This method shows the current user a message by using the growl-component of PrimeFaces.
	 * 
	 * @param severity
	 *            The severity of the message.
	 * @param msg
	 *            The message itself.
	 */
	public static void showMessage(final Severity severity, final String msg) {
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(severity, "", msg));
	}
}
