/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.view;

import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import kieker.analysis.model.analysisMetaModel.MIAnalysisMetaModelFactory;
import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIDisplay;
import kieker.analysis.model.analysisMetaModel.MIFilter;
import kieker.analysis.model.analysisMetaModel.MIInputPort;
import kieker.analysis.model.analysisMetaModel.MIOutputPort;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.MIProperty;
import kieker.analysis.model.analysisMetaModel.MIReader;
import kieker.analysis.model.analysisMetaModel.MIRepository;
import kieker.analysis.model.analysisMetaModel.MIRepositoryConnector;
import kieker.analysis.model.analysisMetaModel.impl.MAnalysisMetaModelFactory;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.plugin.annotation.Property;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.analysis.plugin.reader.AbstractReaderPlugin;
import kieker.analysis.repository.AbstractRepository;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.exception.LibraryLoadException;
import kieker.webgui.common.exception.NewerProjectException;
import kieker.webgui.common.exception.ProjectLoadException;
import kieker.webgui.common.exception.ProjectNotExistingException;
import kieker.webgui.service.IProjectService;
import kieker.webgui.web.beans.application.GlobalPropertiesBean;
import kieker.webgui.web.beans.application.ProjectsBean;
import kieker.webgui.web.beans.session.UserBean;

import org.primefaces.context.RequestContext;
import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.UploadedFile;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

/**
 * The {@link CurrentAnalysisEditorBean} contains the necessary data behind an instance of the analysis editor. It provides various methods to manipulate the current
 * project, as the analysis editor is the most important part of the whole application. The connection to the graph within the editor is done via another bean (the
 * {@link CurrentAnalysisEditorGraphBean}).<br>
 * The class is a Spring managed bean with view scope to make sure that one user (even in one session) can open multiple projects at a time without causing any
 * problems.<br>
 * If editing this class as a programmer keep in mind that the class makes excessive use of java reflection (via {@link ClassAndMethodContainer}) to use the correct
 * classes within every project. It doesn't really use the classes loaded by the WebGUI-ClassLoader. This results also in some methods which returns elements from
 * the type {@link Object} instead of more specific types to avoid incompatibilities between the multiple loaded classes.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("view")
// TODO Remove libraries in a clean way
// TODO Receive libraries once as MIDependencies instead of Strings
// TODO Use a possibility to reload the classloader explicitly - in this case the manager can close it directly in order to avoid too much memory use
public final class CurrentAnalysisEditorBean {
	/**
	 * This field contains the log-object for the whole class, which will be used to log all important errors and exceptions which occur.
	 */
	private static final Log LOG = LogFactory.getLog(CurrentAnalysisEditorBean.class);
	/**
	 * This field contains the factory, which will be used to create the new components for the project.
	 */
	private final MIAnalysisMetaModelFactory factory = new MAnalysisMetaModelFactory();
	/**
	 * This field contains the actual model instance, representing the in-memory model of the current view.
	 */
	private MIProject project;
	/**
	 * This is the corresponding class loader to the project. It contains always the libraries within the lib-folder of the project and additionaly the kieker
	 * library by default.
	 */
	private ClassLoader classLoader;
	/**
	 * This component contains the classes and methods loaded with the specific project class loader.
	 */
	private ClassAndMethodContainer classAndMethodContainer;
	/**
	 * This is the name of the stored project. It can be used as an identifier within the FS-Manager
	 */
	private String projectName;
	/**
	 * This is the time stamp of the moment, the project was loaded or last saved. It can be used to check whether the project has been modified externally in the
	 * meanwhile.
	 */
	private long timeStamp;
	/**
	 * This list contains the available repositories for the current project. Keep in mind: The list doesn't really contain classes from the type
	 * {@code AbstractRepository}, as the repositories are loaded with the specific project class loader. This is just possible as the generic-classes work
	 * internally with {@code Object}.
	 */
	private final List<Class<AbstractRepository>> availableRepositories = Collections.synchronizedList(new ArrayList<Class<AbstractRepository>>());
	/**
	 * This list contains the available filters for the current project. Keep in mind: The list doesn't really contain classes from the type
	 * {@code AbstractFilterPlugin}, as the repositories are loaded with the specific project class loader. This is just possible as the generic-classes work
	 * internally with {@code Object}.
	 */
	private final List<Class<AbstractFilterPlugin>> availableFilters = Collections.synchronizedList(new ArrayList<Class<AbstractFilterPlugin>>());
	/**
	 * This list contains the available readers for the current project. Keep in mind: The list doesn't really contain classes from the type
	 * {@code AbstractReaderPlugin}, as the repositories are loaded with the specific project class loader. This is just possible as the generic-classes work
	 * internally with {@code Object}.
	 */
	private final List<Class<AbstractReaderPlugin>> availableReaders = Collections.synchronizedList(new ArrayList<Class<AbstractReaderPlugin>>());
	/**
	 * This field contains the currently selected component (this can either be a plugin ({@link MIPlugin}) or a repository ({@link MIRepository})).
	 */
	private EObject selectedComponent;
	@Autowired
	private IProjectService projectService;
	@Autowired
	private ProjectsBean projectsBean;
	@Autowired
	private CurrentAnalysisEditorGraphBean currentAnalysisEditorGraphBean;
	@Autowired
	private UserBean userBean;
	@Autowired
	private GlobalPropertiesBean globalPropertiesBean;

	/**
	 * Creates a new instance of this class. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentAnalysisEditorBean() {
		// No code necessary
	}

	/**
	 * This method delivers the project stored in this bean.
	 * 
	 * @return The project for this user.
	 */
	public synchronized MIProject getProject() {
		return this.project;
	}

	/**
	 * This method sets the project stored within this bean and returns the new page for the navigation - depending on the given values.
	 * 
	 * @param newName
	 *            The name of the project.
	 */
	public synchronized void setProjectName(final String newName) {
		// Remember the given parameters
		this.projectName = newName;
	}

	/**
	 * This method initializes the bean by using the current project name to load the project. <b>Do not call this method manually. It will only be accessed by
	 * Spring.</b>
	 */
	public synchronized void initialize() {
		try {
			// Make sure that the initialization will only be done for the init request. During all other requests, the method call have to be ignored.
			if (!FacesContext.getCurrentInstance().isPostback()) {
				// Load the project itself
				this.project = this.projectsBean.openProject(this.projectName);
				// Remember the current time! This is important for the later comparison of the time stamps.
				this.resetTimeStamp();
				// Update the class loader and the specific classes used within various methods in this bean
				this.reloadClassLoader();
				this.reloadClassesAndMethods();
				// Add the libraries within the lib-folder to the current model
				this.initializeModelLibraries();
				// Load the available readers, filters and repositories
				this.initializeToolPalette();
			}
		} catch (final ProjectLoadException ex) {
			CurrentAnalysisEditorBean.LOG.error("An error occured while loading the project.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgProjectLoadingException());
		} catch (final NullPointerException ex) {
			// This exception can occur, when a property has not been initialized
			CurrentAnalysisEditorBean.LOG.error("An error occured while loading the project.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgProjectLoadingException());
		}
	}

	/**
	 * This method reloads the field containing the methods and classes, using the current class loader.
	 * 
	 * @throws ProjectLoadException
	 *             If one or more of the classes and methods necessary for {@link ClassAndMethodContainer} could not be found using the given class loader.
	 */
	private synchronized void reloadClassesAndMethods() throws ProjectLoadException {
		this.classAndMethodContainer = new ClassAndMethodContainer(this.classLoader);
	}

	/**
	 * This method loads the list of available readers, filters and repositories, using the current libraries within the model.
	 */
	private synchronized void initializeToolPalette() {
		// Clean our tool palette
		this.availableFilters.clear();
		this.availableReaders.clear();
		this.availableRepositories.clear();

		// Run through all libraries and add their content to our lists
		final Iterator<MIDependency> dependencies = this.project.getDependencies().iterator();
		while (dependencies.hasNext()) {
			final MIDependency lib = dependencies.next();
			try {
				this.addContentsToToolPalette(lib);
			} catch (final NullPointerException ex) {
				dependencies.remove();
			} catch (final LibraryLoadException ex) {
				CurrentAnalysisEditorBean.LOG.error("A library could not be loaded correctly.", ex);
			}
		}
		try {
			// Run also through the kieker library
			this.addKiekerContentsToToolPalette();
		} catch (final LibraryLoadException ex) {
			CurrentAnalysisEditorBean.LOG.error("A library could not be loaded correctly.", ex);
		}
	}

	/**
	 * This method adds the contents (plugins, repositories) within the given dependency to the tool palette.
	 * 
	 * @param dependency
	 *            The dependency which should be added to the palette.
	 * @throws LibraryLoadException
	 *             If something went wrong during the loading of the library.
	 */
	private synchronized void addContentsToToolPalette(final MIDependency dependency) throws LibraryLoadException {
		this.addContentsToToolPalette(
				this.projectService.getAllPluginsWithinLib(dependency, this.projectName, this.classLoader, this.classAndMethodContainer),
				this.projectService.getAllRepositoriesWithinLib(dependency, this.projectName, this.classLoader, this.classAndMethodContainer));
	}

	/**
	 * This method adds the contents (plugins, repositories) within the kieker dependency to the tool palette.
	 * 
	 * @throws LibraryLoadException
	 *             If something went wrong during the loading of the library.
	 */
	private synchronized void addKiekerContentsToToolPalette() throws LibraryLoadException {
		this.addContentsToToolPalette(
				this.projectService.getAllPluginsWithinKiekerLib(this.classLoader, this.classAndMethodContainer),
				this.projectService.getAllRepositoriesWithinKiekerLib(this.classLoader, this.classAndMethodContainer));
	}

	/**
	 * This method adds all available readers, filters and repositories within the given parameters to the tool palette.
	 * 
	 * @param plugins
	 *            The available readers and filters.
	 * @param repositories
	 *            The available repositories.
	 */
	@SuppressWarnings("unchecked")
	private synchronized void addContentsToToolPalette(final List<Class<AbstractPlugin>> plugins, final List<Class<AbstractRepository>> repositories) {
		// Now run through the available classes and add all non-abstract classes to our lists.
		for (final Class<AbstractRepository> repository : repositories) {
			if (!Modifier.isAbstract(repository.getModifiers())) {
				// Make also sure that the current repository is not "programmaticOnly"
				final Annotation annotationRepository = repository.getAnnotation(this.classAndMethodContainer.getRepositoryAnnotationClass());
				final boolean programmaticOnly = (Boolean) ClassAndMethodContainer.invokeMethod(
						this.classAndMethodContainer.getRepositoryProgrammaticOnlyMethod(),
						annotationRepository, false);
				if (!programmaticOnly) {
					this.availableRepositories.add(repository);
				}
			}
		}

		for (final Class<?> plugin : plugins) {
			// Make sure that the current plugin is neither abstract nor "programmaticOnly"
			final Annotation annotationPlugin = plugin.getAnnotation(this.classAndMethodContainer.getPluginAnnotationClass());
			final boolean programmaticOnly = (Boolean) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPluginProgrammaticOnlyMethod(),
					annotationPlugin, false);
			final boolean isAbstract = Modifier.isAbstract(plugin.getModifiers());

			if (!isAbstract && !programmaticOnly) {
				// The following cast results in the unchecked-cast-warnings, but we know that the cast should be correct.
				if (this.classAndMethodContainer.getAbstractFilterPluginClass().isAssignableFrom(plugin)) {
					this.availableFilters.add((Class<AbstractFilterPlugin>) plugin);
				} else {
					if (this.classAndMethodContainer.getAbstractReaderPluginClass().isAssignableFrom(plugin)) {
						this.availableReaders.add((Class<AbstractReaderPlugin>) plugin);
					}
				}
			}
		}
	}

	/**
	 * This method takes all libraries from the lib-folder and adds them to the in-memory-model.
	 * 
	 * @throws ProjectLoadException
	 *             If something went wrong during the loading of the libraries.
	 */
	private synchronized void initializeModelLibraries() throws ProjectLoadException {
		try {
			final List<String> libs = this.projectService.listAllLibraries(this.projectName);
			// Add them, but remove all existing dependencies so far to avoid double entries. This also makes sure that the model - after it has been opened -
			// points
			// just to valid dependencies (and to all of them).
			this.project.getDependencies().clear();
			for (final String lib : libs) {
				final MIDependency dep = this.factory.createDependency();
				dep.setFilePath(lib);
				this.project.getDependencies().add(dep);
			}
		} catch (final ProjectNotExistingException ex) {
			throw new ProjectLoadException("The project does not exist.", ex);
		}
	}

	/**
	 * This method reloads the class loader. In other words: The class loader will always be able to load classes from the jar-files within the lib-folder of the
	 * project.
	 * 
	 * @throws ProjectLoadException
	 *             If something went wrong.
	 */
	private synchronized void reloadClassLoader() throws ProjectLoadException {
		try {
			this.classLoader = this.projectService.getClassLoader(this.projectName, this); // NOPMD (ClassLoader)
		} catch (final NullPointerException ex) {
			throw new ProjectLoadException("Invalid class loader.", ex);
		} catch (final IOException ex) {
			throw new ProjectLoadException("Could not load classes.", ex);
		} catch (final ProjectNotExistingException ex) {
			throw new ProjectLoadException("Project does not exist.", ex);
		}
	}

	/**
	 * This method sets the time stamp to the current system time.
	 */
	public synchronized void resetTimeStamp() {
		this.timeStamp = System.currentTimeMillis();
	}

	/**
	 * This method delivers the project name stored in this bean.
	 * 
	 * @return The project name for this user.
	 */
	public synchronized String getProjectName() {
		return this.projectName;
	}

	/**
	 * This method delivers the current time stamp.
	 * 
	 * @return The time stamp for this user.
	 */
	public synchronized long getTimeStamp() {
		return this.timeStamp;
	}

	/**
	 * This method can be used to get the description of an {@link AbstractPlugin}- or an {@link AbstractRepository}-class. The description is read via the
	 * annotation using the java reflection API.
	 * 
	 * @param clazz
	 *            The class whose description should be extracted.
	 * @return The description for the class or a substitute if none is available. This is in either case human readable.
	 */
	public synchronized String getDescription(final Class<?> clazz) {
		final String description;
		// Get the two potential annotations
		final Annotation annotationPlugin = clazz.getAnnotation(this.classAndMethodContainer.getPluginAnnotationClass());
		final Annotation annotationRepository = clazz.getAnnotation(this.classAndMethodContainer.getRepositoryAnnotationClass());

		final Method pluginDescrMethod = this.classAndMethodContainer.getPluginDescriptionMethod();
		final Method repoDescrMethod = this.classAndMethodContainer.getRepositoryDescriptionMethod();

		// Now check which one of them is available
		if ((annotationPlugin == null) || ((String) ClassAndMethodContainer.invokeMethod(pluginDescrMethod, annotationPlugin, "")).isEmpty()) {
			if ((annotationRepository == null) || ((String) ClassAndMethodContainer.invokeMethod(repoDescrMethod, annotationRepository, "")).isEmpty()) {
				// None. Deliver a human readable substitute.
				description = "No description available";
			} else {
				description = (String) ClassAndMethodContainer.invokeMethod(repoDescrMethod, annotationRepository, "No description available");
			}
		} else {
			description = (String) ClassAndMethodContainer.invokeMethod(pluginDescrMethod, annotationPlugin, "No description available");
		}

		return description;
	}

	/**
	 * Delivers the content of the dependencies-field of the given component (plugin or repository).
	 * 
	 * @param clazz
	 *            The class whose dependencies should be extracted.
	 * @return The dependencies for the class or a substitute if none is available. This is in either case human readable.
	 */
	public synchronized String getDependencies(final Class<?> clazz) {
		final String description;

		// Get the two potential annotations
		final Annotation annotationPlugin = clazz.getAnnotation(this.classAndMethodContainer.getPluginAnnotationClass());
		final Annotation annotationRepository = clazz.getAnnotation(this.classAndMethodContainer.getRepositoryAnnotationClass());

		final Method pluginDepMethod = this.classAndMethodContainer.getPluginDependenciesMethod();
		final Method repoDepMethod = this.classAndMethodContainer.getRepositoryDependenciesMethod();

		// Now check which one of them is available
		if (annotationPlugin == null) {
			if (annotationRepository == null) {
				// None. Deliver a human readable substitute.
				description = "No dependency information available";
			} else {
				description = (String) ClassAndMethodContainer.invokeMethod(repoDepMethod, annotationRepository, "No dependency information available");
			}
		} else {
			description = (String) ClassAndMethodContainer.invokeMethod(pluginDepMethod, annotationPlugin, "No dependency information available");
		}

		return description;
	}

	/**
	 * Delivers the description of the property of the given component (plugin or repository).
	 * 
	 * @param component
	 *            The component whose property description should be delivered.
	 * @param propertyName
	 *            The name of the property in question.
	 * @return The description of the property.
	 */
	public synchronized String getDescription(final EObject component, final String propertyName) {
		try {
			final String className;
			if (component instanceof MIPlugin) {
				className = ((MIPlugin) component).getClassname();
			} else {
				if (component instanceof MIRepository) {
					className = ((MIRepository) component).getClassname();
				} else {
					// Unknown type
					return "N/A";
				}
			}
			final Class<?> clazz = this.classLoader.loadClass(className);
			final List<Annotation> properties = this.getProperties(clazz);
			// Run through all the properties and find the correct one
			for (final Annotation property : properties) {
				final String name = (String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPropertyNameMethod(), property, "");
				if (propertyName.equals(name)) {
					return (String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPropertyDescriptionMethod(), property, "");
				}
			}
		} catch (final ClassNotFoundException ex) {
			CurrentAnalysisEditorBean.LOG.warn("Could not find class.", ex);
		}
		// If we are here we found nothing
		return "N/A";
	}

	/**
	 * Delivers the properties of the given class, using the annotations and the java reflection API.
	 * 
	 * @param clazz
	 *            The class whose properties will be delivered.
	 * @return A list of properties. It contains instances of {@link Property} - but those from the project class loader.
	 */
	public synchronized List<Annotation> getProperties(final Class<?> clazz) {
		// Get the two potential annotations
		final Annotation annotationPlugin = clazz.getAnnotation(this.classAndMethodContainer.getPluginAnnotationClass());
		final Annotation annotationRepository = clazz.getAnnotation(this.classAndMethodContainer.getRepositoryAnnotationClass());

		final Annotation[] properties;

		// Now check which one of them is available
		if (annotationPlugin == null) {
			if (annotationRepository == null) {
				// None.
				properties = new Property[0];
			} else {
				properties = (Annotation[]) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getRepositoryConfigurationMethod(),
						annotationRepository, new Property[0]);
			}
		} else {
			properties = (Annotation[]) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPluginConfigurationMethod(),
					annotationPlugin, new Property[0]);
		}

		return Arrays.asList(properties);
	}

	/**
	 * Searches for input ports within the given class and returns them.
	 * 
	 * @param clazz
	 *            The class to be analyzed.
	 * @return A list containing the available input ports.
	 */
	public synchronized List<Annotation> getInputPorts(final Class<?> clazz) {
		final List<Annotation> result = new ArrayList<Annotation>();

		for (final Method method : clazz.getMethods()) {
			// Get the potential annotation
			final Annotation annotationPort = method.getAnnotation(this.classAndMethodContainer.getInputPortAnnotationClass());
			// Now check whether it is available
			if (annotationPort != null) {
				result.add(annotationPort);
			}
		}

		return result;
	}

	/**
	 * Searches for output ports within the given class and returns them.
	 * 
	 * @param clazz
	 *            The class to be analyzed.
	 * @return A list containing the available output ports.
	 */
	public synchronized List<Annotation> getOutputPorts(final Class<?> clazz) {
		final List<Annotation> result = new ArrayList<Annotation>();

		// Get the potential annotation
		final Annotation annotationPlugin = clazz.getAnnotation(this.classAndMethodContainer.getPluginAnnotationClass());

		// Now check whether it is available
		if (annotationPlugin != null) {
			for (final Annotation oPort : (Annotation[]) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPluginOutputPortsMethod(),
					annotationPlugin, new Annotation[0])) {
				result.add(oPort);
			}
		}

		return result;
	}

	/**
	 * Searches for repository ports within the given class and returns them.
	 * 
	 * @param clazz
	 *            The class to be analyzed.
	 * @return A list containing the available repository ports.
	 */
	public synchronized List<Annotation> getRepositoryPorts(final Class<?> clazz) {
		final List<Annotation> result = new ArrayList<Annotation>();

		// Get the potential annotation
		final Annotation annotationPlugin = clazz.getAnnotation(this.classAndMethodContainer.getPluginAnnotationClass());

		// Now check whether it is available
		if (annotationPlugin != null) {
			for (final Annotation rPort : (Annotation[]) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPluginRepositoryPortsMethod(),
					annotationPlugin, new Annotation[0])) {
				result.add(rPort);
			}
		}

		return result;
	}

	/**
	 * Searches for displays within the given class and returns them.
	 * 
	 * @param clazz
	 *            The class to be analyzed.
	 * @return A list containing the available displays.
	 */
	public synchronized List<Annotation> getDisplays(final Class<?> clazz) {
		final List<Annotation> result = new ArrayList<Annotation>();

		for (final Method method : clazz.getMethods()) {
			// Get the potential annotation
			final Annotation display = method.getAnnotation(this.classAndMethodContainer.getDisplayAnnotationClass());
			// Now check whether it is available
			if (display != null) {
				result.add(display);
			}
		}

		return result;
	}

	/**
	 * This method is the handler for the file upload. It tries to upload the given file and informs the user via the growl-component.
	 * 
	 * @param event
	 *            The upload event.
	 */
	public synchronized void handleFileUpload(final FileUploadEvent event) {
		// Get the file from the event
		final UploadedFile file = event.getFile();

		try {
			// Use the file system manager to upload the new file
			final MIDependency lib;
			this.projectService.uploadLibrary(file, this.projectName);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, this.globalPropertiesBean.getMsgLibraryUploaded());
			// As it seem to have worked, we can add the library to our model.
			lib = this.factory.createDependency();
			lib.setFilePath(file.getFileName());
			this.project.getDependencies().add(lib);
			// Update our class loader and the available plugins & repositories
			this.reloadClassLoader();
			this.reloadClassesAndMethods();

			// We have to reinitialize the tool palette completely! This is necessary as some of the already existing classes could need the newly loaded classes.
			this.initializeToolPalette();
		} catch (final IOException ex) {
			CurrentAnalysisEditorBean.LOG.error("An error occured while uploading the library.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgLibraryUploadingException());
		} catch (final ProjectLoadException ex) {
			CurrentAnalysisEditorBean.LOG.error("An error occured while uploading the library.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgLibraryUploadingException());
		} catch (final ProjectNotExistingException ex) {
			CurrentAnalysisEditorBean.LOG.error("Project does not exist.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgProjectNotExistingException());
		} catch (final LibraryLoadException ex) {
			CurrentAnalysisEditorBean.LOG.error("An error occured while uploading the library.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgLibraryUploadingException());
		}
	}

	/**
	 * Removes the library with the given name and reloads the classes and the classloader. Furthermore the tool palette will be reinitalized.
	 * 
	 * @param name
	 *            The name of the library to be removed.
	 */
	public synchronized void deleteLibrary(final String name) {
		if (this.projectService.deleteLibrary(this.projectName, name)) {
			// Update our class loader and the available plugins & repositories
			this.reloadClassLoader();
			this.reloadClassesAndMethods();

			// We have to reinitialize the tool palette completely! This is necessary as some of the already existing classes could need the newly loaded classes.
			this.initializeToolPalette();
		}
	}

	/**
	 * This method delivers the available libraries of this project as a pair of strings. The first element is the name of the library, the second one the size in
	 * MiBytes as human readable string. The first element is always the kieker-library.
	 * 
	 * @return The available libraries.
	 */
	public synchronized List<String> getLibraries() {
		try {
			final List<String> result = this.projectService.listAllLibraries(this.projectName);
			result.add(0, "Kieker");

			return result;
		} catch (final ProjectNotExistingException ex) {
			CurrentAnalysisEditorBean.LOG.error("The project does not exist.", ex);
		} catch (final NullPointerException ex) {
			// This exception occurs when a property has not been initialized correctly.
			CurrentAnalysisEditorBean.LOG.error("A null pointer exception occured.", ex);
		}
		return new ArrayList<String>();
	}

	/**
	 * This method delivers the available reader-plugins for the current main project. The delivered plugins are never abstract.
	 * 
	 * @return A list with all readers.
	 */
	public synchronized List<Class<AbstractReaderPlugin>> getAvailableReaders() {
		return this.availableReaders;
	}

	/**
	 * This method delivers the available filter-plugins for the current main project. The delivered plugins are never abstract.
	 * 
	 * @return A list with all filter.
	 */
	public synchronized List<Class<AbstractFilterPlugin>> getAvailableFilters() {
		return this.availableFilters;
	}

	/**
	 * This method delivers the available repositories for the current main project. The delivered repositories are never abstract.
	 * 
	 * @return A list with all repositories.
	 */
	public synchronized List<Class<AbstractRepository>> getAvailableRepositories() {
		return this.availableRepositories;
	}

	/**
	 * This method tries to save the current project and informs the user about success or fail.
	 * 
	 * @param overwriteNewerProject
	 *            This flag determines whether a newer project should be overwritten.
	 */
	public synchronized void saveProject(final boolean overwriteNewerProject) {
		try {
			this.projectService.saveProject(this.projectName, this.project, this.timeStamp, overwriteNewerProject);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_INFO, this.globalPropertiesBean.getMsgProjectSaved());
			// Update the time stamp!
			this.resetTimeStamp();
		} catch (final IOException ex) {
			CurrentAnalysisEditorBean.LOG.error("An error occured while saving the project.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgProjectSavingException());
		} catch (final NewerProjectException ex) {
			CurrentAnalysisEditorBean.LOG.info("The project has been modified externally in the meanwhile.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_WARN, this.globalPropertiesBean.getMsgProjectModified());
			// Give the user the possibility to force-save the project
			RequestContext.getCurrentInstance().execute("forceSaveDlg.show()");
		} catch (final ProjectNotExistingException ex) {
			CurrentAnalysisEditorBean.LOG.error("The project does not exist.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgProjectNotExistingException());
		}
	}

	/**
	 * This method fills the displays of the given plugin. In other words: It tries to extract the list of displays from the given class (using the annotations) and
	 * to convert them into model instances.
	 * 
	 * @param clazz
	 *            The class to be used as a base.
	 * @param plugin
	 *            The plugin to be filled.
	 */
	private synchronized void fillDisplays(final Class<AbstractPlugin> clazz, final MIPlugin plugin) {
		// Get the displays and convert them into model instances
		final List<Annotation> displays = this.getDisplays(clazz);
		for (final Annotation display : displays) {
			final MIDisplay mDisplay = this.factory.createDisplay();
			mDisplay.setName((String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getDisplayNameMethod(), display, "N/A"));
			plugin.getDisplays().add(mDisplay);
		}
	}

	/**
	 * This method fills the ports of the given plugin. In other words: It tries to extract the list of ports from the given class (using the annotations) and
	 * to convert them into model instances.
	 * 
	 * @param clazz
	 *            The class to be used as a base.
	 * @param plugin
	 *            The plugin to be filled.
	 */
	private synchronized void fillPorts(final Class<AbstractPlugin> clazz, final MIPlugin plugin) {
		// Get the ports
		final List<Annotation> inputPorts = this.getInputPorts(clazz);
		final List<Annotation> outputPorts = this.getOutputPorts(clazz);
		final List<Annotation> repositoryPorts = this.getRepositoryPorts(clazz);

		// Add input ports
		if (plugin instanceof MIFilter) {
			for (final Annotation inputPort : inputPorts) {
				final MIInputPort mInputPort = this.factory.createInputPort();
				mInputPort.setName((String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getInputPortNameMethod(), inputPort, "N/A"));
				mInputPort.setParent((MIFilter) plugin);
			}
		}

		// Add output ports.
		for (final Annotation outputPort : outputPorts) {
			final MIOutputPort mOutputPort = this.factory.createOutputPort();
			mOutputPort.setName((String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getOutputPortNameMethod(), outputPort, "N/A"));
			mOutputPort.setParent(plugin);
		}

		// Add repository ports.
		for (final Annotation repositoryPort : repositoryPorts) {
			final MIRepositoryConnector mConnector = this.factory.createRepositoryConnector();
			mConnector.setName((String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getRepositoryPortNameMethod(), repositoryPort,
					"N/A"));
			plugin.getRepositories().add(mConnector);
		}
	}

	/**
	 * This method fills the properties of the given repository or plugin. In other words: It tries to extract the list of properties from the given class (using the
	 * annotations) and to convert them into model instances.
	 * 
	 * @param clazz
	 *            The class to be used as a base.
	 * @param component
	 *            The component to be filled.
	 */
	private synchronized void fillProperties(final Class<?> clazz, final EObject component) {
		// Get the default configuration and use it to initialize the model repository
		final List<Annotation> properties = this.getProperties(clazz);

		for (final Annotation property : properties) {
			final MIProperty mProperty = this.factory.createProperty();

			mProperty.setName((String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPropertyNameMethod(), property, "N/A"));
			mProperty.setValue((String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPropertyDefaultValueMethod(), property, "N/A"));

			if (component instanceof MIPlugin) {
				((MIPlugin) component).getProperties().add(mProperty);
			} else {
				((MIRepository) component).getProperties().add(mProperty);
			}
		}

	}

	/**
	 * This method adds a new repository to the current model, using the given instance of {@code Class} for it.
	 * 
	 * @param clazz
	 *            The class of the repository to be added.
	 */
	public synchronized void addRepository(final Class<AbstractRepository> clazz) {
		try {
			// Create a new instance for the model
			final MIRepository repository = this.factory.createRepository();
			repository.setClassname(clazz.getName());
			repository.setName(clazz.getSimpleName());

			this.fillProperties(clazz, repository);
			// Add it to the project - and to the graph
			this.project.getRepositories().add(repository);
			this.currentAnalysisEditorGraphBean.addRepository(repository);
			this.currentAnalysisEditorGraphBean.refreshGraph();
		} catch (final NoClassDefFoundError ex) {
			// This exception can occur if (for example) a class is missing
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgRepositoryCreationException());
			CurrentAnalysisEditorBean.LOG.error("An error occured during the creation of the repository. Check the dependencies.", ex);
		}
	}

	/**
	 * This method adds a new plugin to the current model, using the given instance of {@code Class} for it.
	 * 
	 * @param clazz
	 *            The class of the plugin to be added. This can be both, a filter or a reader.
	 */
	public synchronized void addPlugin(final Class<AbstractPlugin> clazz) {
		try {
			// Create a new instance for the model
			final MIPlugin plugin;
			if (this.classAndMethodContainer.getAbstractReaderPluginClass().isAssignableFrom(clazz)) {
				plugin = this.factory.createReader();
			} else {
				plugin = this.factory.createFilter();
			}
			plugin.setClassname(clazz.getName());
			plugin.setName(clazz.getSimpleName());

			this.fillProperties(clazz, plugin);
			this.fillPorts(clazz, plugin);
			this.fillDisplays(clazz, plugin);

			// Add it to the project
			this.project.getPlugins().add(plugin);

			// Add the element to the graph
			if (plugin instanceof MIReader) {
				this.currentAnalysisEditorGraphBean.addReader((MIReader) plugin);
			} else {
				this.currentAnalysisEditorGraphBean.addFilter((MIFilter) plugin);
			}
			this.currentAnalysisEditorGraphBean.refreshGraph();
		} catch (final NoClassDefFoundError ex) {
			// This exception can occur if (for example) a class is missing
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, this.globalPropertiesBean.getMsgPluginCreationException());
			CurrentAnalysisEditorBean.LOG.error("An error occured during the creation of the plugin. Check the dependencies.", ex);
		}
	}

	/**
	 * This method delivers the currently selected plugin (or the currently selected repository). If nothing has been selected, null will be returned.
	 * 
	 * @return The currently selected plugin or repository.
	 */
	public synchronized EObject getSelectedPlugin() {
		return this.selectedComponent;
	}

	/**
	 * This method delivers the properties of the currently selected plugin, but it adds also the name- and the class-properties as a string to the list. The first
	 * element is always the classname, the second one is always the pluginname.
	 * 
	 * @return A list with all properties of the plugin plus the name- and class-properties. If no plugin is selected, the list of the selected repository is
	 *         delivered.
	 */
	public synchronized List<Object> getAdvancedPluginProperties() {
		final List<Object> result = new ArrayList<Object>();

		// Add the properties as strings
		result.add("Name");
		result.add("ClassName");

		// Get the original properties of the plugin
		if (this.selectedComponent instanceof MIPlugin) {
			result.addAll(((MIPlugin) this.selectedComponent).getProperties());
		} else {
			result.addAll(((MIRepository) this.selectedComponent).getProperties());
		}

		return result;
	}

	/**
	 * This method delivers a list containing all available filters within the model instance. It is the same as calling project.getPlugins() but instead this method
	 * returns only instances of {@link MIFilter}.
	 * 
	 * @return A list with the available filters.
	 */
	public synchronized List<MIPlugin> getFilters() {
		final EList<MIPlugin> plugins = this.project.getPlugins();
		final List<MIPlugin> filter = new ArrayList<MIPlugin>();

		for (final MIPlugin plugin : plugins) {
			if (plugin instanceof MIFilter) {
				filter.add(plugin);
			}
		}

		return filter;
	}

	/**
	 * Delivers all available repositories.
	 * 
	 * @return A list with all repositories within the model.
	 */
	public synchronized EList<MIRepository> getRepositories() {
		return this.project.getRepositories();
	}

	/**
	 * Delivers the name of the currently selected plugin/repository. This is only necessary for the correct renaming of the components.
	 * 
	 * @return The name of the plugin/repository.
	 */
	public synchronized String getCurrentPluginName() {
		if (this.selectedComponent != null) {
			if (this.selectedComponent instanceof MIPlugin) {
				return ((MIPlugin) this.selectedComponent).getName();
			} else {
				return ((MIRepository) this.selectedComponent).getName();
			}
		} else {
			return "";
		}
	}

	/**
	 * This method should be called if the grid color has been modified.
	 * 
	 * @param event
	 *            The change event.
	 */
	public synchronized void gridColorListener(final ValueChangeEvent event) {
		this.currentAnalysisEditorGraphBean.setGridColor((String) event.getNewValue());
	}

	/**
	 * This method should be called if the grid size has been modified.
	 * 
	 * @param event
	 *            The change event.
	 */
	public synchronized void gridSizeListener(final ValueChangeEvent event) {
		this.currentAnalysisEditorGraphBean.setGridSize((Integer) event.getNewValue());
	}

	/**
	 * This method initializes the modified jit-graph by delivering the necessary javascript commands to the client. It prints all current existing plugins,
	 * repositories and their connections. All further components will be added mostly by other methods.
	 */
	public synchronized void initializeGraph() {
		// Make sure that the currentAnalysisEditorGraphBean knows "this" as well.
		this.currentAnalysisEditorGraphBean.setCurrentAnalysisEditorBean(this);

		// Initialize the graph
		this.currentAnalysisEditorGraphBean.declareGraph();

		// Initialize the reader, filter and repositories
		for (final MIPlugin plugin : this.project.getPlugins()) {
			if (plugin instanceof MIReader) {
				this.currentAnalysisEditorGraphBean.addReader((MIReader) plugin);
			} else {
				this.currentAnalysisEditorGraphBean.addFilter((MIFilter) plugin);
			}
		}

		for (final MIRepository repository : this.project.getRepositories()) {
			this.currentAnalysisEditorGraphBean.addRepository(repository);
		}

		// Now initialize the connections between filters...
		for (final MIPlugin plugin : this.project.getPlugins()) {
			for (final MIOutputPort oPort : plugin.getOutputPorts()) {
				for (final MIInputPort iPort : oPort.getSubscribers()) {
					this.currentAnalysisEditorGraphBean.addConnection(plugin, iPort.getParent(), oPort, iPort);
				}
			}
		}

		// ...and between filters and repositories
		for (final MIPlugin plugin : this.project.getPlugins()) {
			for (final MIRepositoryConnector rPort : plugin.getRepositories()) {
				// It is possible that the connected repository is null, if it hasn't been set yet. Check this.
				if (rPort.getRepository() != null) {
					this.currentAnalysisEditorGraphBean.addConnection(plugin, rPort.getRepository(), rPort);
				}
			}
		}

		this.currentAnalysisEditorGraphBean.initListeners();
		this.currentAnalysisEditorGraphBean.addEdgeConstraints();

		// Now we have to set the default grid size and color of the user
		this.currentAnalysisEditorGraphBean.setGridColor(this.userBean.getGridColor());
		this.currentAnalysisEditorGraphBean.setGridSize(this.userBean.getGridSize());

		// Repaint the graph
		this.currentAnalysisEditorGraphBean.refreshGraph();
	}

	/**
	 * This method should be called if a node (plugin, repository) has been selected.
	 * 
	 * @param node
	 *            The new node to be selected.
	 */
	public synchronized void nodeSelected(final EObject node) {
		this.selectedComponent = node;
	}

	/**
	 * This method should be called if a node (plugin, repository) has been removed.
	 * 
	 * @param node
	 *            The new node to be removed.
	 */
	public synchronized void nodeRemoved(final EObject node) {
		// Remove the component from the project
		if (node instanceof MIPlugin) {
			this.project.getPlugins().remove(node);

			// Remove the corresponding connections
			final List<MIInputPort> toBeRemoved = new ArrayList<MIInputPort>();
			for (final MIPlugin plugin : this.project.getPlugins()) {
				for (final MIOutputPort oPort : plugin.getOutputPorts()) {
					toBeRemoved.clear();
					for (final MIInputPort iPort : oPort.getSubscribers()) {
						if (iPort.getParent() == node) {
							toBeRemoved.add(iPort);
						}
					}
					oPort.getSubscribers().removeAll(toBeRemoved);
				}
			}
		} else {
			this.project.getRepositories().remove(node);

			// Remove the corresponding connections
			for (final MIPlugin plugin : this.project.getPlugins()) {
				for (final MIRepositoryConnector repoConn : plugin.getRepositories()) {
					if (repoConn.getRepository() == node) {
						repoConn.setRepository(null);
					}
				}
			}
		}

		// Unselect the currently selected node if it is the one which has just been removed
		if (this.selectedComponent == node) {
			this.selectedComponent = null; // NOPMD
		}
	}

	/**
	 * This method should be delivered if an edge between two plugins has been created.
	 * 
	 * @param sourcePort
	 *            The source port.
	 * @param targetPort
	 *            The target port.
	 */
	public synchronized void edgeCreated(final MIOutputPort sourcePort, final MIInputPort targetPort) {
		sourcePort.getSubscribers().add(targetPort);
	}

	/**
	 * This method should be delivered if an edge between two plugins has been removed.
	 * 
	 * @param sourcePort
	 *            The source port.
	 * @param targetPort
	 *            The target port.
	 */
	public synchronized void edgeRemoved(final MIOutputPort sourcePort, final MIInputPort targetPort) {
		sourcePort.getSubscribers().remove(targetPort);
	}

	/**
	 * This method should be delivered if an edge between a plugin and a repository has been created.
	 * 
	 * @param sourcePort
	 *            The source port.
	 * @param target
	 *            The target repository.
	 */
	public synchronized void edgeCreated(final MIRepositoryConnector sourcePort, final MIRepository target) {
		sourcePort.setRepository(target);
	}

	/**
	 * This method should be delivered if an edge between a plugin and a repository has been removed.
	 * 
	 * @param sourcePort
	 *            The source port.
	 * @param target
	 *            The target repository.
	 */
	public synchronized void edgeRemoved(final MIRepositoryConnector sourcePort, final MIRepository target) {
		sourcePort.setRepository(null);
	}

}
