/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.view;

import java.util.Map;

import javax.faces.context.FacesContext;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import kieker.analysis.model.analysisMetaModel.MIFilter;
import kieker.analysis.model.analysisMetaModel.MIInputPort;
import kieker.analysis.model.analysisMetaModel.MIOutputPort;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIPort;
import kieker.analysis.model.analysisMetaModel.MIReader;
import kieker.analysis.model.analysisMetaModel.MIRepository;
import kieker.analysis.model.analysisMetaModel.MIRepositoryConnector;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.monitoring.core.registry.Registry;

import org.primefaces.context.RequestContext;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

/**
 * The {@link CurrentAnalysisEditorGraphBean} contains the necessary data behind an graph of the analysis editor. It provides various methods to manipulate the
 * current graph.<br>
 * The class is a Spring managed bean with view scope to make sure that one user (even in one session) can open multiple projects at a time without causing any
 * problems.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("view")
public final class CurrentAnalysisEditorGraphBean {
	/**
	 * This is the log for errors, exceptions etc.
	 */
	private static final Log LOG = LogFactory.getLog(CurrentAnalysisEditorGraphBean.class);
	/**
	 * This is the javascript code to declare the visual graph variable.
	 */
	private static final String JS_CMD_CREATE_GRAPH_VAR = "var graph = GraphFlow();";
	/**
	 * This is the javascript code to add edge constraints to the graph.
	 */
	private static final String JS_CMD_ADD_EDGE_CONSTRAINTS = "graph.addEdgeConstraints()";
	/**
	 * This is the javasscript code to add the click listener to the graph.
	 */
	private static final String JS_CMD_ADD_CLICK_LISTENER = "graph.addListener(\"onClick\", nodeClickListener);";
	/**
	 * This is the javasscript code to add the remove node listener to the graph.
	 */
	private static final String JS_CMD_ADD_REMOVE_NODE_LISTENER = "graph.addListener('onRemoveNode', nodeRemoveListener);";
	/**
	 * This is the javasscript code to add the create edge listener to the graph.
	 */
	private static final String JS_CMD_ADD_CREATE_EDGE_LISTENER = "graph.addListener('onCreateEdge', edgeCreateListener);";
	/**
	 * This is the javasscript code to add the remove edge listener to the graph.
	 */
	private static final String JS_CMD_ADD_REMOVE_EDGE_LISTENER = "graph.addListener('onRemoveEdge', edgeRemoveListener);";
	/**
	 * This is the javascript code for a node object.
	 */
	private static final String JS_CMD_NODE = "{\"id\":\"%s\", \"name\":\"%s\", \"nodeClass\":\"%s\", \"tooltip\":\"%s\"}";
	/**
	 * This is the javascript code for a port object.
	 */
	private static final String JS_CMD_PORT = "{\"name\":\"%s\",\"id\":\"%s\", \"tooltip\":\"%s\"}";
	/**
	 * This is the javascript type name for an input port.
	 */
	private static final String JS_CMD_PORT_TYPE_INPUT = "inputPort";
	/**
	 * This is the javascript code to add a filter to the graph.
	 */
	private static final String JS_CMD_ADD_FILTER = "graph.addNode(%d, %d, %s,[%s],[%s],[%s],'Filter');";
	/**
	 * This is the javascript code to add a reader to the graph.
	 */
	private static final String JS_CMD_ADD_READER = "graph.addNode(%d, %d, %s,[%s],null,[%s], 'Reader');";
	/**
	 * This is the javascript code to add a repository to the graph.
	 */
	private static final String JS_CMD_ADD_REPOSITORY = "graph.addNode(%d, %d, %s, null, [%s], null, 'Repository');";
	/**
	 * This is the javascript code to add an edge to the graph.
	 */
	private static final String JS_CMD_ADD_EDGE = "graph.addEdge(\"%s\", \"%s\", \"%s\");";
	/**
	 * This is the javascript code to redraw the command.
	 */
	private static final String JS_CMD_REFRESH_GRAPH = "graph.refresh();";
	/**
	 * This is the javascript code to rename a node within the graph.
	 */
	private static final String JS_CMD_RENAME_NODE = "graph.getNode('%s').name = '%s';";
	/**
	 * This is the javascript code to enable the grid within the graph.
	 */
	private static final String JS_CMD_ENABLE_GRID = "graph.setGridVisible(true);";
	/**
	 * This is the javascript code to disable the grid within the graph.
	 */
	private static final String JS_CMD_DISABLE_GRID = "graph.setGridVisible(false);";
	/**
	 * This is the javascript code to enable the grid snap within the graph.
	 */
	private static final String JS_CMD_ENABLE_SNAP = "graph.setGridSnap(true);";
	/**
	 * This is the javascript code to disable the grid snap within the graph.
	 */
	private static final String JS_CMD_DISABLE_SNAP = "graph.setGridSnap(false);";
	/**
	 * This is the javascript code to use the scale-to-fit-function.
	 */
	private static final String JS_CMD_SCALE_TO_FIT = "graph.scaleToFit();";
	/**
	 * This is the javascript code the set the color of the grid.
	 */
	private static final String JS_CMD_SET_GRID_COLOR = "graph.setGridColor('%s');";
	/**
	 * This is the javascript code the set the size of the grid.
	 */
	private static final String JS_CMD_SET_GRID_SIZE = "graph.setGridSize(%d);";
	/**
	 * This is the name of the (only) input port of every repository.
	 */
	private static final Object REPOSITORY_INPUT_PORT = "R";

	private boolean gridEnabled = false; // NOPMD (The field IS accessed later)
	private boolean snapEnabled = false; // NOPMD (The field IS accessed later)

	/**
	 * This map contains all components (plugins, repositories and ports) within the graph to identify them with a unique ID.
	 */
	private final Registry<EObject> componentMap = new Registry<EObject>();

	private CurrentAnalysisEditorBean currentAnalysisEditorBean;

	/**
	 * Creates a new instance of this class. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentAnalysisEditorGraphBean() {
		// No code necessary
	}

	/**
	 * Declares the the graph variable.
	 */
	public synchronized void declareGraph() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_CREATE_GRAPH_VAR);
		RequestContext.getCurrentInstance().execute("graph.setNodeIcon('Filter', '../img/graphIcons/FilterIcon.png');");
		RequestContext.getCurrentInstance().execute("graph.setNodeIcon('Reader', '../img/graphIcons/ReaderIcon.png');");
		RequestContext.getCurrentInstance().execute("graph.setNodeIcon('Repository', '../img/graphIcons/RepositoryIcon.png');");
	}

	/**
	 * Initializes the listeners for the graph.
	 */
	public synchronized void initListeners() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_CLICK_LISTENER);
		RequestContext.getCurrentInstance().execute("graph.addListener('onError', function(e){Log.write(e);});");
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_REMOVE_NODE_LISTENER);
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_CREATE_EDGE_LISTENER);
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_REMOVE_EDGE_LISTENER);
	}

	/**
	 * Adds the edge constraints to the graph.
	 */
	public synchronized void addEdgeConstraints() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_EDGE_CONSTRAINTS);
	}

	/**
	 * Refreshes the graph after changes have been made.
	 */
	public synchronized void refreshGraph() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_REFRESH_GRAPH);
	}

	/**
	 * This method adds a filter to the current graph.
	 * 
	 * @param filter
	 *            The filter which should be added to the graph.
	 */
	public synchronized void addFilter(final MIFilter filter) {
		RequestContext.getCurrentInstance().execute(
				String.format(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_FILTER, 0, 0, this.assembleGraphString(filter),
						this.assembleGraphRepositoryPortString(filter.getRepositories()), this.assembleGraphInputPortString(filter),
						this.assembleGraphOutputPortString(filter)));
	}

	/**
	 * This method adds a reader to the current graph.
	 * 
	 * @param reader
	 *            The reader which should be added to the graph.
	 */
	public synchronized void addReader(final MIReader reader) {
		RequestContext.getCurrentInstance().execute(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_READER, 0, 0, this.assembleGraphString(reader),
				this.assembleGraphRepositoryPortString(reader.getRepositories()), this.assembleGraphOutputPortString(reader)));
	}

	/**
	 * This method adds a repository to the current graph.
	 * 
	 * @param repository
	 *            The repository which should be added to the graph.
	 */
	public synchronized void addRepository(final MIRepository repository) {
		final String repoPort = String.format(CurrentAnalysisEditorGraphBean.JS_CMD_PORT, CurrentAnalysisEditorGraphBean.JS_CMD_PORT_TYPE_INPUT,
				CurrentAnalysisEditorGraphBean.REPOSITORY_INPUT_PORT, "N/A");
		RequestContext.getCurrentInstance().execute(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_REPOSITORY, 0, 0,
				this.assembleGraphString(repository),
				repoPort));
	}

	/**
	 * This method adds a connection between two filters to the graph.
	 * 
	 * @param source
	 *            The source plugin.
	 * @param destination
	 *            The destination plugin.
	 * @param outputPort
	 *            The output port.
	 * @param inputPort
	 *            The input port.
	 */
	public synchronized void addConnection(final MIPlugin source, final MIPlugin destination, final MIOutputPort outputPort, final MIInputPort inputPort) {
		RequestContext.getCurrentInstance().execute(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_EDGE, this.assembleGraphPortID(source, outputPort),
				this.assembleGraphPortID(destination, inputPort), ""));
	}

	/**
	 * This method adds a connection between a filter and a repository to the graph.
	 * 
	 * @param source
	 *            The source plugin.
	 * @param destination
	 *            The destination repository.
	 * @param port
	 *            The repository port.
	 */
	public synchronized void addConnection(final MIPlugin source, final MIRepository destination, final MIRepositoryConnector port) {
		RequestContext.getCurrentInstance().execute(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_ADD_EDGE, this.assembleGraphPortID(source, port),
				this.assembleGraphPortID(destination), ""));
	}

	/**
	 * This method assembles the string containing the given ports.
	 * 
	 * @param ports
	 *            The ports to be used within the string command.
	 * @return A string containing the JS commands to create the ports.
	 */
	private synchronized String assembleGraphPortString(final EList<? extends MIPort> ports) {

		final StringBuilder builder = new StringBuilder();
		final int len = ports.size();

		for (int i = 0; i < len; i++) {
			final MIPort port = ports.get(i);

			if (i != 0) {
				builder.append(',');
			}

			builder.append(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_PORT, port.getName(), this.componentMap.get(port), port.getName()));
		}

		return builder.toString();

	}

	/**
	 * This method assembles the string containing the available repository ports of the given plugin.
	 * 
	 * @param ports
	 *            The ports which will be used.
	 * @return A string containing the JS commands to create the repository ports.
	 */
	private synchronized String assembleGraphRepositoryPortString(final EList<MIRepositoryConnector> ports) {

		final StringBuilder builder = new StringBuilder();
		final int len = ports.size();

		for (int i = 0; i < len; i++) {
			final MIRepositoryConnector port = ports.get(i);

			if (i != 0) {
				builder.append(',');
			}

			builder.append(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_PORT, port.getName(), this.componentMap.get(port), port.getName()));
		}

		return builder.toString();
	}

	/**
	 * This method assembles the string containing the available input ports of the given filter.
	 * 
	 * @param filter
	 *            The filter whose input ports will be used.
	 * @return A string containing the JS commands to create the input ports.
	 */
	private synchronized String assembleGraphInputPortString(final MIFilter filter) {
		return this.assembleGraphPortString(filter.getInputPorts());
	}

	/**
	 * This method assembles the string containing the available output ports of the given plugin.
	 * 
	 * @param plugin
	 *            The plugin whose output ports will be used.
	 * @return A string containing the JS commands to create the output ports.
	 */
	private synchronized String assembleGraphOutputPortString(final MIPlugin plugin) {
		return this.assembleGraphPortString(plugin.getOutputPorts());
	}

	/**
	 * This method assembles the ID of the port for the graph based on the given parameters.
	 * 
	 * @param plugin
	 *            The parent plugin of the port.
	 * @param port
	 *            The port itself.
	 * @return The ID for the port within the graph
	 */
	private synchronized String assembleGraphPortID(final MIPlugin plugin, final MIPort port) {
		return this.componentMap.get(plugin) + "." + this.componentMap.get(port);
	}

	/**
	 * This method assembles the ID of the port for the graph based on the given parameters.
	 * 
	 * @param plugin
	 *            The parent plugin of the port.
	 * @param port
	 *            The port itself.
	 * @return The ID for the port within the graph
	 */
	private synchronized Object assembleGraphPortID(final MIPlugin plugin, final MIRepositoryConnector port) {
		return this.componentMap.get(plugin) + "." + this.componentMap.get(port);
	}

	/**
	 * This method assembles the ID of the port for the graph based on the given parameters.
	 * 
	 * @param repository
	 *            The parent repository of the port.
	 * @return The ID for the port within the graph
	 */
	private synchronized Object assembleGraphPortID(final MIRepository repository) {
		return this.componentMap.get(repository) + "." + CurrentAnalysisEditorGraphBean.REPOSITORY_INPUT_PORT;
	}

	/**
	 * Assembles a human-readable string of the given repository, which can also be used as a graph ID.
	 * 
	 * @param repository
	 *            The repository whose ID should be delivered.
	 * @return A human readable ID.
	 */
	private synchronized String assembleGraphString(final MIRepository repository) {
		final String className = repository.getClassname();
		final String shortName = className.substring(className.lastIndexOf('.') + 1);
		return String.format(CurrentAnalysisEditorGraphBean.JS_CMD_NODE, this.componentMap.get(repository), repository.getName(), shortName, className);
	}

	/**
	 * Assembles a human-readable string of the given plugin, which can also be used as a graph ID.
	 * 
	 * @param plugin
	 *            The plugin whose ID should be delivered.
	 * @return A human readable ID.
	 */
	private synchronized String assembleGraphString(final MIPlugin plugin) {
		final String className = plugin.getClassname();
		final String shortName = className.substring(className.lastIndexOf('.') + 1);
		return String.format(CurrentAnalysisEditorGraphBean.JS_CMD_NODE, this.componentMap.get(plugin), plugin.getName(), shortName, className);
	}

	/**
	 * This is the action which can be called from the javascript code to show that a node has been clicked. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public synchronized void nodeClicked() {
		// Get the parameters
		final Map<String, String> paramMap = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		final String clickedNodeID = paramMap.get("ID");

		// Now search the correct node
		try {
			final EObject selectedNode = this.componentMap.get(Integer.parseInt(clickedNodeID));
			if ((selectedNode != null) && (this.currentAnalysisEditorBean != null)) {
				this.currentAnalysisEditorBean.nodeSelected(selectedNode);
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.warn("Invalid ID", ex);
		}
	}

	/**
	 * This is the action which can be called from the javascript code to show that a node has been removed. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public synchronized void nodeRemoved() {
		// Get the parameters
		final Map<String, String> paramMap = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		final String clickedNodeID = paramMap.get("ID");
		// Now search the correct node
		try {
			final EObject selectedNode = this.componentMap.get(Integer.parseInt(clickedNodeID));
			if ((selectedNode != null) && (this.currentAnalysisEditorBean != null)) {
				this.currentAnalysisEditorBean.nodeRemoved(selectedNode);
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
		}
	}

	/**
	 * This is the action which can be called from the javascript code to show that an edge has been created. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public synchronized void edgeCreated() {
		// Get the parameters
		final Map<String, String> paramMap = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();

		final String sourcePortID = paramMap.get("sourcePortID").split("\\.")[1];
		final String targetPortID = paramMap.get("targetPortID").split("\\.")[1];

		// Now search the correct components
		try {
			final EObject sourcePort = this.componentMap.get(Integer.parseInt(sourcePortID));
			if (CurrentAnalysisEditorGraphBean.REPOSITORY_INPUT_PORT.equals(targetPortID)) {
				// This is a special case: An edge between a filter and a repository
				final String targetID = paramMap.get("targetPortID").split("\\.")[0];
				final EObject targetRepo = this.componentMap.get(Integer.parseInt(targetID));

				if ((sourcePort instanceof MIRepositoryConnector) && (targetRepo instanceof MIRepository) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.edgeCreated((MIRepositoryConnector) sourcePort, (MIRepository) targetRepo);
				}
			} else {
				// This is the normal case: An edge between two filters
				final EObject targetPort = this.componentMap.get(Integer.parseInt(targetPortID));

				if ((sourcePort != null) && (targetPort != null) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.edgeCreated((MIOutputPort) sourcePort, (MIInputPort) targetPort);
				}
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
		}
	}

	/**
	 * This is the action which can be called from the javascript code to show that an edge has been removed. It informs the connected
	 * {@link CurrentAnalysisEditorBean} about this.
	 */
	public synchronized void edgeRemoved() {
		// Get the parameters
		final Map<String, String> paramMap = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();

		final String sourcePortID = paramMap.get("sourcePortID").split("\\.")[1];
		final String targetPortID = paramMap.get("targetPortID").split("\\.")[1];

		// Now search the correct components
		try {
			final EObject sourcePort = this.componentMap.get(Integer.parseInt(sourcePortID));
			if (CurrentAnalysisEditorGraphBean.REPOSITORY_INPUT_PORT.equals(targetPortID)) {
				// This is a special case: An edge between a filter and a repository
				final String targetID = paramMap.get("targetPortID").split("\\.")[0];
				final EObject targetRepo = this.componentMap.get(Integer.parseInt(targetID));
				if ((sourcePort instanceof MIRepositoryConnector) && (targetRepo instanceof MIRepository) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.edgeRemoved((MIRepositoryConnector) sourcePort, (MIRepository) targetRepo);
				}
			} else {
				// This is the normal case: An edge between two filters
				final EObject targetPort = this.componentMap.get(Integer.parseInt(targetPortID));

				if ((sourcePort != null) && (targetPort != null) && (this.currentAnalysisEditorBean != null)) {
					this.currentAnalysisEditorBean.edgeRemoved((MIOutputPort) sourcePort, (MIInputPort) targetPort);
				}
			}
		} catch (final NumberFormatException ex) {
			// Ignore an invalid ID, but log it.
			CurrentAnalysisEditorGraphBean.LOG.info("Invalid ID", ex);
		}
	}

	/**
	 * This method sets the color of the grid within the graph.
	 * 
	 * @param color
	 *            The new color as a HTML color (FF0080 e.g.).
	 */
	public synchronized void setGridColor(final String color) {
		final String cmd = String.format(CurrentAnalysisEditorGraphBean.JS_CMD_SET_GRID_COLOR, "#" + color);
		RequestContext.getCurrentInstance().execute(cmd);
		this.refreshGraph();
	}

	/**
	 * This method sets the size of the grid within the graph.
	 * 
	 * @param size
	 *            The new grid size.
	 */
	public synchronized void setGridSize(final int size) {
		final String cmd = String.format(CurrentAnalysisEditorGraphBean.JS_CMD_SET_GRID_SIZE, size);
		RequestContext.getCurrentInstance().execute(cmd);
		this.refreshGraph();
	}

	/**
	 * This method scales the graph to fit the current view. After this method-call, all components of the graph should be visible.
	 */
	public synchronized void scaleToFit() {
		RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_SCALE_TO_FIT);
	}

	/**
	 * This method switches the visibility of the grid within the graph.
	 */
	public synchronized void switchGrid() {
		if (this.gridEnabled) {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_DISABLE_GRID);
		} else {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ENABLE_GRID);
		}

		this.gridEnabled ^= true; // this.gridEnabled = !this.gridEnabled;
		this.refreshGraph();
	}

	/**
	 * This method switches the grid snap within the graph.
	 */
	public synchronized void switchSnap() {
		if (this.snapEnabled) {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_DISABLE_SNAP);
		} else {
			RequestContext.getCurrentInstance().execute(CurrentAnalysisEditorGraphBean.JS_CMD_ENABLE_SNAP);
		}

		this.snapEnabled ^= true; // this.snapEnabled = !this.snapEnabled;
	}

	/**
	 * Getter for the property {@link CurrentAnalysisEditorGraphBean#gridEnabled}.
	 * 
	 * @return The current value of the property.
	 */
	public synchronized boolean isGridEnabled() {
		return this.gridEnabled;
	}

	/**
	 * Getter for the property {@link CurrentAnalysisEditorGraphBean#snapEnabled}.
	 * 
	 * @return The current value of the property.
	 */
	public synchronized boolean isSnapEnabled() {
		return this.snapEnabled;
	}

	/**
	 * The setter for the field {@link CurrentAnalysisEditorGraphBean#currentAnalysisEditorBean}.
	 * 
	 * @param currentAnalysisEditorBean
	 *            The new value for this field.
	 */
	public synchronized void setCurrentAnalysisEditorBean(final CurrentAnalysisEditorBean currentAnalysisEditorBean) {
		this.currentAnalysisEditorBean = currentAnalysisEditorBean;
	}

	/**
	 * Renames a given node and repaints the graph.
	 * 
	 * @param node
	 *            The node to rename.
	 * @param newName
	 *            The new name of the node.
	 */
	public synchronized void renameNode(final EObject node, final String newName) {
		RequestContext.getCurrentInstance().execute(String.format(CurrentAnalysisEditorGraphBean.JS_CMD_RENAME_NODE, this.componentMap.get(node), newName));
		this.refreshGraph();
	}

}
