/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.web.beans.view;

import java.io.IOException;
import java.util.List;

import javax.faces.application.Application;
import javax.faces.application.FacesMessage;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.context.FacesContext;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import kieker.analysis.model.analysisMetaModel.MIDisplayConnector;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.MIView;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.exception.DisplayNotFoundException;
import kieker.webgui.common.exception.ProjectNotExistingException;
import kieker.webgui.service.IProjectService;
import kieker.webgui.web.beans.application.GlobalPropertiesBean;
import kieker.webgui.web.beans.application.ProjectsBean;

import org.primefaces.component.dashboard.Dashboard;
import org.primefaces.component.panel.Panel;
import org.primefaces.model.DashboardColumn;
import org.primefaces.model.DashboardModel;
import org.primefaces.model.DefaultDashboardColumn;
import org.primefaces.model.DefaultDashboardModel;

/**
 * The {@link CurrentCockpitBean} contains the necessary data behind an instance of the cockpit. It provides methods to read the state of the currently selected
 * project.<br>
 * The class is a Spring managed bean with view scope to make sure that one user (even in one session) can open multiple projects at a time without causing any
 * problems.
 * 
 * @author Nils Christian Ehmke
 */
@Component
@Scope("view")
public final class CurrentCockpitBean {

	private static final int NUMBER_COLUMNS = 2;
	private static final Log LOG = LogFactory.getLog(CurrentCockpitBean.class);
	@Autowired
	private IProjectService projectService;
	private String projectName;
	private MIProject project;
	private MIView activeView;
	private ClassAndMethodContainer classAndMethodContainer;
	private Dashboard dashboard;
	private DashboardModel dashboardModel;
	@Autowired
	private ProjectsBean projectsBean;

	/**
	 * Creates a new instance of this class. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentCockpitBean() {
		this.createDashboard();
	}

	/**
	 * Creates the initial dashboard object.
	 */
	private synchronized void createDashboard() {
		final FacesContext fc = FacesContext.getCurrentInstance();
		final Application application = fc.getApplication();

		// Create the primefaces dashboard
		this.dashboard = (Dashboard) application.createComponent(fc, "org.primefaces.component.Dashboard", "org.primefaces.component.DashboardRenderer");
		this.dashboard.setId("dashboard");

		// Create the model and add the columns
		this.dashboardModel = new DefaultDashboardModel();
		for (int i = 0; i < CurrentCockpitBean.NUMBER_COLUMNS; i++) {
			final DashboardColumn column = new DefaultDashboardColumn();
			this.dashboardModel.addColumn(column);
		}
		this.dashboard.setModel(this.dashboardModel);
	}

	/**
	 * Fills the initial dashboard object.
	 */
	private synchronized void fillDashboard() {
		// Dump the old entries
		this.clearDashboard();

		// Now add the entries from the current view
		if (this.activeView != null) {
			final FacesContext fc = FacesContext.getCurrentInstance();
			final Application application = fc.getApplication();

			// Add a panel for every display connector we have
			final List<MIDisplayConnector> connectors = this.activeView.getDisplayConnectors();
			long currId = 0;
			for (final MIDisplayConnector connector : connectors) {
				final Panel panel = (Panel) application.createComponent(fc, "org.primefaces.component.Panel", "org.primefaces.component.PanelRenderer");
				panel.setId("displayConnectorPanel_" + currId);
				panel.setHeader(connector.getName());
				panel.setClosable(false);
				panel.setToggleable(false);

				this.getDashboard().getChildren().add(panel);
				final DashboardColumn column = this.dashboard.getModel().getColumn(0);
				column.addWidget(panel.getId());
				final HtmlOutputText text = new HtmlOutputText();
				text.setValue("N/A");

				panel.getChildren().add(text);
				currId++;
			}
		}
	}

	/**
	 * Clears the dashboard and removes all children within it.
	 */
	private synchronized void clearDashboard() {
		// Run through all columns of the dashboard and remove the items
		final List<DashboardColumn> columns = this.dashboard.getModel().getColumns();
		for (final DashboardColumn column : columns) {
			column.getWidgets().clear();
		}
		// Now run through the dashboard itself and remove the items as well
		this.dashboard.getChildren().clear();
	}

	/**
	 * This method delivers the project stored in this bean.
	 * 
	 * @return The project for this user.
	 */
	public synchronized MIProject getProject() {
		return this.project;
	}

	/**
	 * This method sets the project stored within this bean.
	 * 
	 * @param newName
	 *            The name of the project.
	 */
	public synchronized void setProjectName(final String newName) {
		// Remember the given parameters
		this.projectName = newName;
	}

	/**
	 * This method initializes the bean by using the current project name to load the project. <b>Do not call this method manually. It will only be accessed by
	 * Spring.</b>
	 */
	public synchronized void initalize() {
		// TODO Replace the log messages with the localizes messages
		try {
			// Make sure that the initialization will only be done for the init request.
			if (!FacesContext.getCurrentInstance().isPostback()) {
				// Remember the given parameters
				this.project = this.projectsBean.openProject(this.projectName);

				if (this.project != null) {
					final ClassLoader classLoader = this.projectService.getClassLoader(this.projectName, this); // NOPMD (ClassLoader)
					this.classAndMethodContainer = new ClassAndMethodContainer(classLoader);
					this.fillDashboard();
				}
			}
		} catch (final ProjectNotExistingException ex) {
			CurrentCockpitBean.LOG.error("An error occured while loading the project.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
		} catch (final IOException ex) {
			CurrentCockpitBean.LOG.error("An error occured while loading the project.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
		} catch (final NullPointerException ex) {
			// This exception occurs, when the projectsBean has not been initialized
			CurrentCockpitBean.LOG.error("An error occured while loading the project.", ex);
			GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
		}
	}

	/**
	 * This method delivers the project name stored in this bean.
	 * 
	 * @return The project name for this user.
	 */
	public synchronized String getProjectName() {
		return this.projectName;
	}

	/**
	 * Delivers the current content for the given display as plain text.
	 * 
	 * @param displayName
	 *            The name of the display.
	 * @return The current content of the display, if it exists. If the display does not exist or the active view is not set 'N/A' will be returned. If an access to
	 *         the display fails, "Error" will be returned.
	 */
	public synchronized String updatePlainTextDisplay(final String displayName) {
		if ((this.activeView != null) && (this.projectName != null)) {
			try {
				final Object displayObj = this.projectService.getDisplay(this.projectName, this.activeView.getName(), displayName);
				final String result = (String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPlainTextgetTextMethod(), displayObj,
						"Error");
				return result;
			} catch (final DisplayNotFoundException ex) {
				CurrentCockpitBean.LOG.warn("Display not found.", ex);
			}
		}
		return "N/A";
	}

	/**
	 * Delivers the current content for the given display as html text.
	 * 
	 * @return Currently this method returns only N/A.
	 */
	public synchronized String updateHtmlTextDisplay() {
		return "N/A";
	}

	/**
	 * Delivers the current content for the given display as Image link.
	 * 
	 * @return Currently this method returns only N/A.
	 */
	public synchronized String updateImageDisplay() {
		return "N/A";
	}

	/**
	 * Delivers the currently active view.
	 * 
	 * @return The active view.
	 */
	public synchronized MIView getActiveView() {
		return this.activeView;
	}

	/**
	 * Sets the active view to a new value.
	 * 
	 * @param activeView
	 *            The new active view.
	 */
	public synchronized void setActiveView(final MIView activeView) {
		this.activeView = activeView;

		this.fillDashboard();
	}

	/**
	 * Getter for the property {@link CurrentCockpitEditorBean#dashboard}.
	 * 
	 * @return The current value of the property.
	 */
	public synchronized Dashboard getDashboard() {
		return this.dashboard;
	}

	/**
	 * Setter for the property {@link CurrentCockpitEditorBean#dashboard}.
	 * 
	 * @param dashboard
	 *            The new value for the property.
	 */
	public synchronized void setDashboard(final Dashboard dashboard) {
		this.dashboard = dashboard;
		this.dashboard.setModel(this.dashboardModel);
	}
}
